<?php

namespace App\Core\Elastic;

use App\Core\Elastic\Category\CategoryIndex;
use App\Core\Elastic\Product\ProductIndex;
use Elasticsearch\Client;

/**
 * Class IndexFactory
 * @package App\Core\Elastic
 */
class IndexFactory
{
    /** @var string Константа для работы с индексом товаров */
    public const TYPE_PRODUCT = 'product';

    /** @var string Константа для работы с индексом категорий */
    public const TYPE_CATEGORY = 'category';

    /** @var Client Клиент Эластика */
    private $client;

    /**
     * IndexFactory constructor.
     * @param Client $client
     */
    public function __construct(Client $client)
    {
        $this->client = $client;
    }

    /**
     * @param string $type
     * @return Index|null
     */
    public function index(string $type): ?Index
    {
        switch ($type) {
            case self::TYPE_PRODUCT:
                return new ProductIndex($this->client);
            case self::TYPE_CATEGORY:
                return new CategoryIndex($this->client);
            default:
                return null;
        }
    }

    /**
     * Получает массив имен всех доступных индексов.
     *
     * @return string[]
     */
    public static function getIndexesNamesList(): array
    {
        return [
            static::TYPE_PRODUCT,
            static::TYPE_CATEGORY,
        ];
    }
}
