<?php

namespace App\Core\Elastic\Product\Indexing\Actions;

use App\Core\Elastic\Product\Indexing\OfferTicket;
use App\Core\Elastic\Product\Indexing\ProcessesTicket;
use App\Core\Elastic\Product\Indexing\ProductTicket;
use Ensi\OffersClient\Api\OffersApi;
use Ensi\OffersClient\Dto\Offer;
use Ensi\OffersClient\Dto\OfferSaleStatusEnum;
use Ensi\OffersClient\Dto\SearchOffersRequest;
use Illuminate\Support\Collection;

class CreateOffersAction
{
    use ProcessesTicket;

    public function __construct(private OffersApi $api)
    {
    }

    public function execute(Collection $tickets): Collection
    {
        $productIds = $tickets->map(fn(ProductTicket $ticket) => $ticket->source->id)->all();

        $sourceOffers = $this->loadOffers($productIds)->groupBy('product_id');

        $offers = new Collection();

        $this->forEachTicket(
            $tickets,
            fn(ProductTicket $ticket) => $this->fillOffer($ticket, $offers, $sourceOffers->get($ticket->id))
        );

        return $offers;
    }

    private function loadOffers(array $productIds): Collection
    {
        $request = (new SearchOffersRequest())
            ->setFilter((object)['product_id' => $productIds]);

        return collect($this->api->searchOffers($request)->getData());
    }

    /**
     * @param ProductTicket $productTicket
     * @param Collection|OfferTicket[] $tickets
     * @param Collection|Offer[]|null $offers
     */
    private function fillOffer(ProductTicket $productTicket, Collection $tickets, ?Collection $offers): void
    {
        if ($offers === null || $offers->isEmpty()) {
            return;
        }

        foreach ($offers as $offer) {
            $ticket = OfferTicket::create($offer->getId(), $productTicket);
            $target = $ticket->target;

            $target->sellerId = $offer->getSellerId();
            $target->storageAddress = $offer->getStorageAddress();
            $target->setPrice([
                'price' => $offer->getBasePrice(),
                'cost' => $offer->getBasePrice(),
            ]);

            if ($offer->getSaleStatus() !== OfferSaleStatusEnum::ON_SALE) {
                $target->resetSaleActive();;
            }

            $tickets->put($ticket->id, $ticket);
        }
    }
}
