<?php

namespace App\Core\Elastic\Product\Indexing;

use App\Core\Elastic\Indexer;
use App\Core\Elastic\Product\ProductIndex;
use Illuminate\Support\Collection;

class PipelineIndexer implements Indexer
{
    /** @var Collection|ProductTicket[] */
    private Collection $tickets;

    public function __construct(private ProductsPipeline $pipeline, private ProductIndex $index)
    {
        $this->tickets = new Collection();
    }

    public function index(int $id): bool
    {
        return !($this->getTicket($id)?->checkFailed() ?? true);
    }

    public function preload(array $ids): void
    {
        if (!$ids) {
            return;
        }

        $this->tickets = ProductTicket::load($ids);

        $this->executePipeline();

        $this->save();
    }

    private function executePipeline(): void
    {
        $this->tickets
            ->filter(fn(ProductTicket $ticket) => $ticket->source !== null)
            ->whenNotEmpty(fn(Collection $tickets) => $this->pipeline->execute($tickets));
    }

    private function save(): void
    {
        $this->tickets
            ->filter(fn(ProductTicket $ticket) => !$ticket->isFailed)
            ->map(fn(ProductTicket $ticket) => $ticket->toOperation())
            ->whenNotEmpty(fn(Collection $operations) => $this->index->bulk($operations));
    }

    private function getTicket(int $id): ?ProductTicket
    {
        return $this->tickets->get($id);
    }
}
