<?php

namespace App\Core\Elastic\Product\Indexing;

use App\Core\Elastic\Product\ProductIndexOperation;
use App\Core\Elastic\Product\ProductIndexRecord;
use App\Domain\Products\Models\Product;
use Exception;
use Illuminate\Support\Collection;
use LogicException;

class ProductTicket
{
    public bool $isFailed = false;
    public ?Exception $error = null;

    public function __construct(public int $id, public ProductIndexRecord $target, public ?Product $source)
    {
    }

    public static function load(array $productIds): Collection
    {
        $products = Product::query()
            ->with([
                'category',
                'brand',
                'manufacturer',
                'country',
                'properties',
                'properties.property',
                'properties.propertyDirectoryValue',
                'images',
            ])
            ->whereIn('id', $productIds)
            ->get()
            ->keyBy('id');

        return collect($productIds)
            ->mapWithKeys(fn(int $id) => [$id => new self($id, new ProductIndexRecord(), $products->get($id))]);
    }

    public function toOperation(): ProductIndexOperation
    {
        if ($this->isFailed) {
            throw new LogicException('Невозможно индексировать необработанный товар');
        }

        return $this->source === null
            ? new ProductIndexOperation($this->id, 'delete', null)
            : new ProductIndexOperation($this->id, 'index', $this->target);
    }

    public function fail(?Exception $error = null): void
    {
        $this->isFailed = true;
        $this->error = $error;
    }

    public function checkFailed(): bool
    {
        if (!$this->isFailed) {
            return false;
        }

        if ($this->error !== null) {
            throw $this->error;
        }

        return true;
    }
}
