<?php

namespace App\Core\Elastic\Product\Indexing;

use App\Core\Elastic\Product\Indexing\Stages\AttributesStage;
use App\Core\Elastic\Product\Indexing\Stages\ImagesStage;
use App\Core\Elastic\Product\Indexing\Stages\OffersStage;
use App\Core\Elastic\Product\Indexing\Stages\ProductGroupsStage;
use App\Core\Elastic\Product\Indexing\Stages\ProductInfoStage;
use App\Support\Log\DefaultLoggerAwareInterface;
use Exception;
use Illuminate\Support\Collection;
use Psr\Log\LoggerAwareTrait;

class ProductsPipeline implements DefaultLoggerAwareInterface
{
    use LoggerAwareTrait;

    public function __construct(
        private ProductInfoStage $infoStage,
        private AttributesStage $attributesStage,
        private ImagesStage $imagesStage,
        private ProductGroupsStage $productGroupsStage,
        private OffersStage $offersStage
    ) {
    }

    public function execute(Collection $tickets): void
    {
        try {
            $this->infoStage->execute($tickets);
            $this->attributesStage->execute($tickets);
            $this->imagesStage->execute($tickets);
            $this->productGroupsStage->execute($tickets);
            $this->offersStage->execute($tickets);
        } catch (Exception $exception) {
            $this->logger?->error("В процессе индексирования произошла ошибка. {$exception->getMessage()}");

            $tickets->each(fn(ProductTicket $ticket) => $ticket->fail($exception));
        }
    }
}
