<?php

namespace App\Core\Elastic\Product\Indexing\Stages;

use App\Core\Elastic\Product\Indexing\PipelineStageInterface;
use App\Core\Elastic\Product\Indexing\ProcessesTicket;
use App\Core\Elastic\Product\Indexing\ProductTicket;
use App\Domain\Products\Models\Product;
use Illuminate\Support\Collection;

class AttributesStage implements PipelineStageInterface
{
    use ProcessesTicket;

    /**
     * @inheritDoc
     */
    public function execute(Collection $tickets): void
    {
        $this->forEachTicket($tickets, $this);
    }

    public function __invoke(ProductTicket $ticket): void
    {
        $product = $ticket->source;
        $target = $ticket->target;

        $characteristics = [];
        foreach ($product->properties as $propertyValue) {
            $property = $propertyValue->property;

            if ($propertyValue->propertyDirectoryValue) {
                $propValue = $propertyValue->propertyDirectoryValue->code;
                $propVisibleValue = $propertyValue->propertyDirectoryValue->name;
            } else {
                $propValue = $propertyValue->value($property->type);
                $propVisibleValue = $propValue;
            }

            if ($property->is_filterable) {
                $target->props[] = "{$property->code}#{$propValue}";
            }

            if (!isset($characteristics[$property->name])) {
                $characteristics[$property->name] = [];
            }

            $characteristics[$property->name][] = $propVisibleValue;
        }

        $this->fillCharacteristics($product, $characteristics);

        foreach ($characteristics as $name => $values) {
            $target->addCharacteristic($name, $values);
        }
    }

    /**
     * Обогащает характеристики данными из товара.
     *
     * @param Product $product
     * @param array $characteristics
     */
    protected function fillCharacteristics(Product $product, array &$characteristics)
    {
        if ($product->calories) {
            $characteristics['Калорийность'][] = $product->calories;
        }
        if ($product->proteins) {
            $characteristics['Белки на 100 г, г'][] = $product->proteins;
        }
        if ($product->fats) {
            $characteristics['Жиры на 100 г, г'][] = $product->fats;
        }
        if ($product->carbohydrates) {
            $characteristics['Углеводы на 100 г, г'][] = $product->carbohydrates;
        }
        if ($product->weight) {
            $characteristics['Масса нетто, кг'][] = $product->weight;
        }
        if ($product->weight_gross) {
            $characteristics['Масса брутто, кг'][] = $product->weight_gross;
        }
        if ($product->ingredients) {
            $characteristics['Ингредиенты'][] = $product->ingredients;
        }
        if ($product->length || $product->width || $product->height) {
            $characteristics['ДxШxВ, мм'][] = ($product->length ?? '-') . 'x' . ($product->width ?? '-') . 'x' . ($product->height ?? '-');
        }
        if ($product->manufacturer) {
            $characteristics['Производитель'][] = $product->manufacturer->name;
        }
        if ($product->country) {
            $characteristics['Страна производства'][] = $product->country->name;
        }
    }
}
