<?php

namespace App\Core\Elastic\Product\Indexing\Stages;

use App\Core\Elastic\Product\Indexing\PipelineStageInterface;
use App\Core\Elastic\Product\Indexing\ProcessesTicket;
use App\Core\Elastic\Product\Indexing\ProductTicket;
use App\Core\Elastic\Product\ProductIndex;
use App\Core\Elastic\Product\ProductIndexRecord;
use App\Domain\Categories\Models\Category;
use App\Domain\Classifiers\Models\Brand;
use App\Domain\Products\Models\ProductArchiveType;
use Illuminate\Support\Collection;

class ProductInfoStage implements PipelineStageInterface
{
    use ProcessesTicket;

    /**
     * @inheritDoc
     */
    public function execute(Collection $tickets): void
    {
        $this->forEachTicket($tickets, $this);
    }

    public function __invoke(ProductTicket $ticket): void
    {
        $target = $ticket->target;
        $product = $ticket->source;

        $target->name = $product->name;
        $target->code = $product->code;
        $target->barcode = $product->barcode;
        $target->externalId = $product->external_id;
        $target->variantGroupId = $product->variant_group_id;

        $target->dateAdd = $product->created_at;
        $target->weight = $product->weight_gross;

        $this->setNew($target, $product->is_new);
        $this->setBrand($target, $product->brand);
        $this->setArchive($target, $product->archive);
        $this->setCategory($target, $product->category);
    }

    private function setNew(ProductIndexRecord $target, bool $value): void
    {
        if (!$value) {
            return;
        }

        $target->isNew = true;
        $target->tags[] = ProductIndex::TAG_NEW;
    }

    private function setBrand(ProductIndexRecord $target, ?Brand $brand): void
    {
        if (!$brand) {
            return;
        }

        $target->brandName = $brand->name;
        $target->brandCode = $brand->code;
    }

    private function setArchive(ProductIndexRecord $target, ProductArchiveType $archive): void
    {
        $target->archive = $archive->value;

        if ($archive->isArchive()) {
            $target->addStrike('В архиве');
        }
    }

    private function setCategory(ProductIndexRecord $target, ?Category $category): void
    {
        if (!$category) {
            $target->addStrike("Нет категории");
            return;
        }

        $target->categoryName = $category->name;
        $codes = $category->ancestors->pluck('code')->all();
        $codes[] = $category->code;
        $target->categoryCodes = $codes;
    }
}
