<?php

namespace App\Core\Elastic\Product;

/**
 * Class OfferIndexProperty
 * @package App\Core\Elastic\Product
 */
class OfferIndexProperty
{
    /** @var array Описание цены */
    public $price = [];
    /** @var int Наличие на складе */
    public $qty;
    /** @var int Идентификатор склада */
    public $storeId;
    /** @var int Идентификатор оффера */
    public $offerId;
    /** @var int Идентификатор продавца */
    public $sellerId;
    /** @var string Адрес хранения товара в магазине */
    public $storageAddress;
    /** @var bool Активность оффера (вычисляется на основе остальных полей) */
    private $active = true;
    /** @var bool Активность продаж (ручная галочка, выставляется в админке) */
    private $saleActive = true;
    /** @var array Страйки */
    private $strikes = [];

    public function addStrike(string $msg): void
    {
        $this->active = false;
        $this->strikes[] = $msg;
    }

    public function resetSaleActive()
    {
        $this->saleActive = false;
    }

    /**
     * @param $price
     */
    public function setPrice($price)
    {
        $this->price = $price;
    }

    /**
     * Оффер активен?
     *
     * @return bool
     */
    public function isActive(): bool
    {
        return (bool)$this->active;
    }

    /**
     * @return array
     */
    public function toArray(): array
    {
        return [
            'active' => $this->active,
            'saleActive' => $this->saleActive,

            'price' => $this->price,
            'qty' => $this->qty,
            'store_id' => $this->storeId,
            'offer_id' => $this->offerId,
            'seller_id' => $this->sellerId,
            'storage_address' => $this->storageAddress,

            'strikes' => $this->strikes,
        ];
    }
}
