<?php

namespace App\Core\Elastic\Product;

use App\Core\Elastic\Extractor;

/**
 * Class ProductExtractor - класс который преобразует запись индекса к массиву с запрошенными полями
 * @package App\Core\Elastic\Product
 */
class ProductArrayExtractor implements Extractor
{
    /** @var array */
    private $fields;
    private $sellersIds;
    private $sellersIdsFlip;
    private $offersIds;
    private $offersIdsFlip;

    public function __construct(array $fields, $seller_id, $offer_id)
    {
        $this->fields = $fields;
        $this->sellersIds = $seller_id
            ? (is_array($seller_id) ? $seller_id : [$seller_id])
            : [];
        $this->sellersIdsFlip = array_flip($this->sellersIds);
        $this->offersIds = $offer_id
            ? (is_array($offer_id) ? $offer_id : [$offer_id])
            : [];
        $this->offersIdsFlip = array_flip($this->offersIds);
    }

    public function extract(array $source): array
    {
        // берем первый оффер, подходящий под заданные seller_id или offer_id
        $offerKey = 0;
        foreach ($source["_source"]["offers"] as $k => $offer) {
            if ($this->sellersIdsFlip && isset($offer['seller_id']) && isset($this->sellersIdsFlip[$offer['seller_id']])) {
                $offerKey = $k;

                break;
            }
            if ($this->offersIdsFlip && isset($offer['offer_id']) && isset($this->offersIdsFlip[$offer['offer_id']])) {
                $offerKey = $k;

                break;
            }
        }
        $result = [];
        foreach ($this->fields as $field) {
            switch ($field) {
                case "id":
                    $value = $source["_id"];

                    break;
                case "dateAdd":
                    $value = $source["_source"]["date_add"];

                    break;
                case "catalogImage":
                    $value = $source["_source"]["catalog_image"];

                    break;
                case "barcode":
                    $value = $source["_source"]["barcode"];

                    break;
                case "externalId":
                    $value = $source["_source"]["external_id"];

                    break;
                case "categoryName":
                    $value = data_get($source, "_source.category.name");

                    break;
                case "categoryCode":
                    $chain = data_get($source, "_source.category.code");
                    $value = end($chain);

                    break;
                case "categoryChain":
                    $value = data_get($source, "_source.category.code");

                    break;
                case "brandName":
                    $value = data_get($source, "_source.brand.name");

                    break;
                case "brandCode":
                    $value = data_get($source, "_source.brand.code");

                    break;
                case "cost":
                    $value = data_get($source, "_source.offers.{$offerKey}.price.cost");

                    break;
                case "price":
                    $value = data_get($source, "_source.offers.{$offerKey}.price.price");

                    break;
                case "discounts":
                    $value = data_get($source, "_source.offers.{$offerKey}.price.discounts");

                    break;
                case "offerId":
                    $value = data_get($source, "_source.offers.{$offerKey}.offer_id");

                    break;
                case "sellerId":
                    $value = data_get($source, "_source.offers.{$offerKey}.seller_id");

                    break;
                case "storageAddress":
                    $value = data_get($source, "_source.offers.{$offerKey}.storage_address");

                    break;
                case "storeId":
                    $value = data_get($source, "_source.offers.{$offerKey}.store_id");

                    break;
                case "qty":
                    $value = data_get($source, "_source.offers.{$offerKey}.qty");

                    break;
                case "active":
                    $productActive = data_get($source, "_source.active");
                    $offerActiveKey = $offerKey ? "_source.offers.{$offerKey}.active" : "_source.has_active_offers";
                    $offerActive = data_get($source, $offerActiveKey);
                    $value = $productActive && $offerActive;

                    break;
                case 'productActive':
                    $value = data_get($source, "_source.active");

                    break;
                case "saleActive":
                    $value = data_get($source, "_source.offers.{$offerKey}.saleActive");

                    break;
                case "isNew":
                    $value = in_array(ProductIndex::TAG_NEW, data_get($source, "_source.tags") ?? []);

                    break;
                case "variantGroup":
                    $value = data_get($source, "_source.variant_group");

                    break;
                default:
                    $value = $source["_source"][$field] ?? null;
            }
            $result[$field] = $value;
        }

        return $result;
    }

    /**
     * @param array $data
     */
    public function extractAggregation(array $data)
    {
        if (!isset($data["aggregations"]) || !$data["aggregations"]) {
            return null;
        }
        $result = [];
        foreach ($data["aggregations"][ProductQuery::CATEGORY_COUNT_AGGREGATION]["buckets"] as $bucket) {
            $result[ProductQuery::CATEGORY_COUNT_AGGREGATION][$bucket["key"]] = $bucket["doc_count"];
        }

        return $result;
    }
}
