<?php

namespace App\Core\Rest\Controller;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\HttpKernel\Exception\HttpException;

/**
 * Trait CreateAction - rest-действие для создания ресурса.
 * @package Common\Rest\Controller
 */
trait CreateAction
{
    use RestAction;
    use WriteAction;

    public function create(Request $request)
    {
        /** @var Model $modelClass */
        $modelClass = $this->modelClass();
        $data = $request->only($this->writableFieldList());
        if ($this->isInvalid($data)) {
            throw new BadRequestHttpException($this->validationErrors->first());
        }
        /** @var Model $model */
        $model = new $modelClass($data);
        $ok = $model->save();

        if (!$ok) {
            throw new HttpException(500);
        }

        /** @noinspection PhpUndefinedFieldInspection */
        return response()->json([
            'id' => $model->id,
        ], 201);
    }
}
