<?php

namespace App\Core\Rest\Controller;

use App\Core\Rest\RestQuery;
use App\Core\Rest\RestSerializable;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Trait ListAction - rest-действие для получения списка ресурсов.
 * @package Common\Rest\Controller
 */
trait ReadAction
{
    use RestAction;

    public function read(Request $request)
    {
        /** @var Model|RestSerializable $modelClass */
        $modelClass = $this->modelClass();

        $restQuery = new RestQuery($request);

        $query = $modelClass::query();
        $channelConditions = $restQuery->getFilter('channel');
        if ($channelConditions) {
            [$op, $value] = current($channelConditions);
            $query->where('channel_id', $value[0]);
            $query->orWhereNull('channel_id');
            $restQuery->removeFilter('channel');
        }

        $id = $request->route('id');
        if ($id) {
            $query = $modelClass::modifyQuery($query->where('id', $id), $restQuery);

            /** @var RestSerializable $model */
            $model = $query->first();
            if (!$model) {
                throw new NotFoundHttpException();
            }
            $items = $model->toRest($restQuery);
        } else {
            // todo добавить право для просмотра списка
            $pagination = $restQuery->getPage();
            $baseQuery = $query;
            if ($pagination) {
                $baseQuery->offset($pagination['offset'])->limit($pagination['limit']);
            }
            $query = $modelClass::modifyQuery($baseQuery, $restQuery);
            $items = $query->get()
                ->map(function (RestSerializable $model) use ($restQuery) {
                    return $model->toRest($restQuery);
                });
        }

        return response()->json([
            'items' => $items,
        ]);
    }
}
