<?php

namespace App\Core\Rest\Controller;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Trait UpdateAction - rest-действие для обновления ресурса.
 * @package Common\Rest\Controller
 */
trait UpdateAction
{
    use RestAction;
    use WriteAction;

    public function update(int $id, Request $request)
    {
        /** @var Model $modelClass */
        $modelClass = $this->modelClass();
        $data = $request->only($this->writableFieldList());

        if ($this->isInvalid($data)) {
            throw new BadRequestHttpException($this->validationErrors->first());
        }

        $model = $modelClass::query()->where('id', $id)->first();
        if (!$model) {
            throw new NotFoundHttpException();
        }
        $model->fill($data);
        $ok = $model->save();
        if (!$ok) {
            throw new HttpException(500);
        }

        return response('', 204);
    }
}
