<?php

namespace App\Core\Rest\Controller;

use Illuminate\Support\Facades\Validator;
use Illuminate\Support\MessageBag;

trait WriteAction
{
    /** @var MessageBag */
    protected $validationErrors;

    /**
     * Получить список полей, которые можно редактировать через стандартные rest действия.
     * Пример return ['name', 'status'];
     * @return array
     */
    abstract protected function writableFieldList(): array;

    /**
     * Получить список правил проверки данных, записываемых в ресурс.
     * @return array
     */
    protected function inputValidators(): array
    {
        return [];
    }

    protected function isInvalid(array $data): bool
    {
        $validator = Validator::make($data, $this->inputValidators(), [
            'required' => ':attribute is required',
            'in' => ':attribute value not in valid array',
        ]);
        $fails = $validator->fails();
        if ($fails) {
            $this->validationErrors = $validator->errors();
        }

        return $fails;
    }
}
