<?php

namespace App\Domain\Categories\Actions\DirectoryValues;

use App\Domain\Categories\Models\Property;
use App\Domain\Categories\Models\PropertyDirectoryValue;
use App\Domain\Categories\Models\PropertyType;
use App\Domain\Support\Concerns\AppliesToAggregate;
use App\Domain\Support\Models\Model;
use App\Exceptions\ActionNotSupportedException;
use Closure;

trait AppliesToDirectory
{
    use AppliesToAggregate;

    protected function createModel(): Model
    {
        return new Property();
    }

    protected function applyToDirectory(int $valueId, Closure $action): mixed
    {
        return $this->transaction(function () use ($valueId, $action) {
            $value = PropertyDirectoryValue::findOrFail($valueId);

            return $this->apply($value->property_id, function (Property $property) use ($value, $action) {
                if ($property->type->isNot(PropertyType::DIRECTORY())) {
                    throw new ActionNotSupportedException("Свойство $property->name не является справочником");
                }

                return $action($property, $value);
            });
        });
    }
}
