<?php

namespace App\Domain\Categories\Models;

use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\DB;

/**
 * @property bool $is_inherited
 * @property bool $is_required
 * @property int $category_id
 * @property int $property_link_id
 * @property-read Collection|PropertyDirectoryValue[] $directory
 *
 * @method static Builder|self forCategory(int $categoryId)
 */
class CategoryPropertyView extends Property
{
    protected $fillable = [];

    public function __construct(array $attributes = [])
    {
        $this->mergeCasts([
            'is_inherited' => 'bool',
            'is_required' => 'bool',
            'category_id' => 'int',
            'property_link_id' => 'int',
        ]);

        parent::__construct($attributes);
    }

    public function directory(): HasMany
    {
        return $this->hasMany(PropertyDirectoryValue::class, 'property_id', 'id');
    }

    public function scopeForCategory(Builder $query, int $categoryId): Builder
    {
        $linkIds = CategoryPropertyLink::selectEffectiveIds($categoryId);
        $subQuery = CategoryPropertyLink::query()
            ->select([
                'property_id',
                'category_id',
                'is_required',
                'id as property_link_id',
                DB::raw("CASE WHEN category_id=$categoryId THEN false ELSE true END as is_inherited"),
            ])
            ->whereIn('id', $linkIds);

        return $query->joinSub($subQuery, 'link', 'properties.id', '=', 'link.property_id');
    }
}
