<?php

namespace App\Domain\Categories\Models;

use App\Domain\Support\Models\Model;
use Cviebrock\EloquentSluggable\Sluggable;
use Database\Factories\Categories\PropertiesFactory;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Support\Collection;

/**
 * Класс-модель для сущности "Свойства (характеристики) товара".
 *
 * @property string $name - название (для админок)
 * @property string $display_name - отображаемое в публичке название
 * @property string $code - код
 * @property PropertyType $type - тип (string, integer, double, datetime, directory)
 * @property bool $is_multiple - множественность
 * @property bool $is_filterable - показывать в фильтре
 * @property bool $is_color - хранит цвет
 *
 * @property Collection|PropertyDirectoryValue[] $directory
 */
class Property extends Model
{
    use Sluggable;

    /**
     * @var array Заполняемые поля модели
     */
    const FILLABLE = ['name', 'display_name', 'code', 'type', 'is_multiple', 'is_filterable', 'is_color'];

    protected $fillable = self::FILLABLE;
    protected $table = 'properties';

    protected $casts = [
        'type' => PropertyType::class,
        'is_multiple' => 'bool',
        'is_filterable' => 'bool',
    ];

    public function categoryPropertyLinks(): HasMany
    {
        return $this->hasMany(CategoryPropertyLink::class);
    }

    public function directory(): HasMany
    {
        return $this->hasMany(PropertyDirectoryValue::class);
    }

    public function sluggable(): array
    {
        return [
            'code' => [
                'source' => 'name',
                'separator' => '_',
                'unique' => true,
                'maxLength' => 25,
            ],
        ];
    }

    public static function factory(): PropertiesFactory
    {
        return PropertiesFactory::new();
    }
}
