<?php

namespace App\Domain\Categories\Models;

use App\Domain\Support\Models\Model;
use Cviebrock\EloquentSluggable\Sluggable;
use Database\Factories\Categories\DirectoryValueFactory;

/**
 * Класс-модель для сущности "Варианты значений для свойств-справочников".
 *
 * @property int $property_id - id свойства
 * @property string $name - название
 * @property string $code - символьный код для фильтрации
 */
class PropertyDirectoryValue extends Model
{
    use Sluggable;

    const FILLABLE = ['name', 'code'];

    protected $fillable = self::FILLABLE;
    protected $table = 'property_directory_values';

    public function sluggable(): array
    {
        return [
            'code' => [
                'source' => 'name',
                'separator' => '_',
                'unique' => true,
                'maxLength' => 20,
            ],
        ];
    }

    public static function factory(): DirectoryValueFactory
    {
        return DirectoryValueFactory::new();
    }
}
