<?php

namespace App\Domain\Classifiers\Models;

use App\Domain\Support\Models\Model;
use Cviebrock\EloquentSluggable\Sluggable;
use Database\Factories\Classifiers\BrandsFactory;

/**
 * Brand - бренд производителя.
 *
 * @property string $name название
 * @property string $code ЧПУ код
 * @property string $description описание бренда
 * @property string|null $file картинка
 */
class Brand extends Model
{
    use Sluggable;

    public const FILLABLE = ['name', 'code', 'description'];

    /** @var string */
    protected $table = 'brands';
    protected $fillable = self::FILLABLE;

    /**
     * @return array
     */
    public function sluggable(): array
    {
        return [
            'code' => [
                'source' => 'name',
                'separator' => '_',
                'unique' => true,
                'maxLength' => 30,
            ],
        ];
    }

    public static function factory(): BrandsFactory
    {
        return BrandsFactory::new();
    }
}
