<?php

namespace App\Domain\Products\Actions\Attributes;

use App\Domain\Products\Models\Product;
use App\Domain\Support\Concerns\AppliesToAggregate;
use App\Domain\Support\Models\Model;
use App\Support\Actions\ActionDecorator;
use App\Support\Actions\DecoratesAction;

class PatchProductAttributesAction
{
    use AppliesToAggregate;
    use DecoratesAction;

    protected bool $clearUnassigned = false;

    public function __construct(protected SetProductAttributesAction $setAttributesAction)
    {
    }

    public function execute(int $productId, array $values): Product
    {
        return $this->apply($productId, function (Product $product) use ($values) {
            $this->setAttributesAction->execute($product, $values, $this->clearUnassigned);
            return $product;
        });
    }

    /**
     * Обогащает результат загрузкой связанных значений.
     * @return ActionDecorator|$this
     */
    public function enrichResult(): static|ActionDecorator
    {
        return $this->decorateResult(fn(Product $product) => $product->load('properties'));
    }

    protected function createModel(): Model
    {
        return new Product();
    }
}
