<?php

namespace App\Domain\Products\Actions\Images;

use App\Domain\Products\Models\Product;
use App\Domain\Products\Models\ProductImage;
use App\Domain\Support\Concerns\AppliesToAggregate;
use App\Domain\Support\Models\Model;
use Illuminate\Support\Arr;

class CreateProductImageAction
{
    use AppliesToAggregate;

    public function execute(array $fields): ProductImage
    {
        return $this->transaction(
            fn () => $this->createImage((int)$fields['product_id'], $fields)
        );
    }

    protected function createImage(int $productId, array $fields): ProductImage
    {
        return $this->apply($productId, function (Product $product) use ($fields) {
            $image = new ProductImage();
            $image->fill(Arr::only($fields, $image->getFillable()));

            $this->saveRelatedOrThrow($product->images(), $image);
            return $image;
        });
    }

    protected function createModel(): Model
    {
        return new Product();
    }
}
