<?php

namespace App\Domain\Products\Actions\Images;

use App\Domain\Products\Models\Product;
use App\Domain\Products\Models\ProductImage;
use App\Domain\Support\Concerns\AppliesToAggregate;
use App\Domain\Support\Models\Model;
use Illuminate\Support\Arr;

class PatchProductImageAction
{
    use AppliesToAggregate;

    public function execute(int $imageId, array $fields): ProductImage
    {
        return $this->transaction(
            fn () => $this->patchImageFields(ProductImage::findOrFail($imageId), $fields)
        );
    }

    protected function patchImageFields(ProductImage $image, array $fields): ProductImage
    {
        $this->apply($image->product_id, function () use ($image, $fields) {
            $image->fill(Arr::only($fields, $image->getFillable()));
            $this->saveOrThrow($image);
        });

        return $image;
    }

    protected function createModel(): Model
    {
        return new Product();
    }

}
