<?php

namespace App\Domain\Products\Actions\Images;

use App\Domain\Products\Models\Product;
use App\Domain\Products\Models\ProductImage;
use App\Domain\Support\Actions\BaseFileAction;
use App\Domain\Support\Models\Model;
use App\Support\Files\FileIntent;
use Illuminate\Http\UploadedFile;

class UploadProductImageAction extends BaseFileAction
{
    public function execute(int $imageId, ?UploadedFile $file): ProductImage
    {
        return $this->transaction(function () use ($imageId, $file) {
            $image = ProductImage::findOrFail($imageId);
            return $this->attachToImage($image, $file);
        });
    }

    protected function createIntent(int $productId, UploadedFile $file): FileIntent
    {
        return FileIntent::create($file)
            ->inDirectory('products')
            ->withNamePrefix("product_$productId");
    }

    protected function createModel(): Model
    {
        return new Product();
    }

    private function attachToImage(ProductImage $image, ?UploadedFile $file): ProductImage
    {
        return $this->attach(
            $image->product_id,
            $this->createIntent($image->product_id, $file),
            fn (Product $product, string $filePath) => $this->setImageFile($image, $filePath)
        );
    }

    private function setImageFile(ProductImage $image, string $filePath): void
    {
        $image->file = $filePath;

        $this->saveOrThrow($image);

        $this->pushReturnValue($image);
    }
}
