<?php

namespace App\Domain\Products\Actions\Products;

use App\Core\Elastic\ScheduleRunner;
use App\Domain\Products\Models\Product;
use Greensight\LaravelPhpRdKafkaConsumer\Exceptions\KafkaConsumerException;
use Illuminate\Support\Facades\Validator;
use Laravie\SerializesQuery\Eloquent;
use RdKafka\Message;
use Throwable;

class PrepareMarkProductsForIndexAction
{
    /** @throws KafkaConsumerException */
    public function execute(Message $message): void
    {
        $rules = [
            'ids' => ['required', 'array'],
            'ids.*' => ['integer'],
        ];

        try {
            if (! $payload = $this->validatePayload($rules, $message->payload)) {
                return;
            }

            $serializedQuery = Eloquent::serialize(
                Product::query()
                    ->whereIn('id', $payload['ids'])
            );

            (new MarkProductsForIndexAction(new ScheduleRunner()))
                ->execute($serializedQuery);
        } catch (Throwable $e) {
            $message = 'An error occurred while saving the message,'
                . " kafka message payload: {$message->payload}\n"
                . $e->getMessage();
            throw new KafkaConsumerException($message);
        }
    }

    /** @throws KafkaConsumerException */
    public function validatePayload(array $rules, string $data): ?array
    {
        if (empty($data)) {
            return null;
        }

        $payload = json_decode($data, true);

        try {
            return Validator::make($payload, $rules)->validate();
        } catch (Throwable $e) {
            throw new KafkaConsumerException("Kafka message validation fail\n"
                . "Payload: {$data}\n"
                . "Error message: "
                . $e->getMessage());
        }
    }
}
