<?php

namespace App\Domain\Products\Actions\Products;

use App\Domain\Products\Models\Product;
use App\Domain\Support\Concerns\AppliesToAggregate;
use App\Domain\Support\Models\Model;
use function data_combine_assoc;

class ReplaceProductAction
{
    use AppliesToAggregate;

    public function execute(int $productId, array $fields): Product
    {
        return $this->updateOrCreate($productId, function (Product $product) use ($fields) {
            $fieldsWithDefaults = array_merge([
                'archive' => 1,
                'weight' => 0,
                'weight_gross' => 0,
                'height' => 0,
                'length' => 0,
                'width' => 0,
                'is_new' => false,
            ], $fields);

            $product->fill(
                data_combine_assoc($product->getFillable(), $fieldsWithDefaults)
            );
        });
    }

    protected function createModel(): Model
    {
        return new Product();
    }
}
