<?php

namespace App\Domain\Products\Models;

use App\Domain\Support\Models\Model;
use Database\Factories\Products\ProductImagesFactory;
use Illuminate\Database\Eloquent\Builder;

/**
 * ProductImage картинка товара.
 *
 * @property int $id
 * @property int $code //Deprecated
 * @property ProductImageType $type Тип картинки
 * @property string $file
 * @property int $product_id
 */
class ProductImage extends Model
{
    const FILLABLE = ['type'];

    public const TYPE_MAIN = 1;
    public const TYPE_CATALOG = 2;
    public const TYPE_GALLERY = 3;
    public const TYPE_DESCRIPTION = 4;

    protected $fillable = self::FILLABLE;

    protected $casts = [
        'product_id' => 'int',
    ];

    public function getTypeAttribute(): ProductImageType
    {
        return ProductImageType::fromValue($this->attributes['code'] ?? ProductImageType::GALLERY);
    }

    /**
     * @param ProductImageType|int $value
     */
    public function setTypeAttribute($value): void
    {
        $this->attributes['code'] = ProductImageType::serializeDatabase(ProductImageType::fromValue($value));
    }

    public function scopeWithType(Builder $query, $type): Builder
    {
        return $query->where('code', $type);
    }

    public static function factory(array $attributes = []): ProductImagesFactory
    {
        return ProductImagesFactory::new($attributes);
    }
}
