<?php

namespace App\Http\ApiV1\Modules\Categories\Controllers;

use App\Domain\Categories\Actions\Categories\CreateCategoryAction;
use App\Domain\Categories\Actions\Categories\DeleteCategoryAction;
use App\Domain\Categories\Actions\Categories\DeleteCategoryImageAction;
use App\Domain\Categories\Actions\Categories\DeleteManyCategoriesAction;
use App\Domain\Categories\Actions\Categories\PatchCategoryAction;
use App\Domain\Categories\Actions\Categories\ReplaceCategoryAction;
use App\Domain\Categories\Actions\Categories\SaveCategoryImageAction;
use App\Http\ApiV1\Modules\Categories\Queries\CategoriesQuery;
use App\Http\ApiV1\Modules\Categories\Queries\CategoriesTreeQuery;
use App\Http\ApiV1\Modules\Categories\Queries\CategoryPropertiesQueryFactory;
use App\Http\ApiV1\Modules\Categories\Requests\CreateOrReplaceCategoryRequest;
use App\Http\ApiV1\Modules\Categories\Requests\PatchCategoryRequest;
use App\Http\ApiV1\Modules\Categories\Requests\UploadCategoryImageRequest;
use App\Http\ApiV1\Modules\Categories\Resources\CategoriesResource;
use App\Http\ApiV1\Modules\Categories\Resources\CategoriesTreeResourceCollection;
use App\Http\ApiV1\Modules\Categories\Resources\CategoryPropertiesResource;
use App\Http\ApiV1\Support\Pagination\PageBuilderFactory;
use App\Http\ApiV1\Support\Requests\MassDeleteRequest;
use App\Http\ApiV1\Support\Resources\EmptyResource;
use Illuminate\Http\Request;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;

class CategoriesController
{
    public function search(PageBuilderFactory $pageBuilderFactory, CategoriesQuery $query): AnonymousResourceCollection
    {
        return CategoriesResource::collectPage(
            $pageBuilderFactory->fromQuery($query)->build()
        );
    }

    public function get(int $entityId, CategoriesQuery $query): CategoriesResource
    {
        return new CategoriesResource($query->findOrFail($entityId));
    }

    public function create(CreateOrReplaceCategoryRequest $request, CreateCategoryAction $action): CategoriesResource
    {
        return new CategoriesResource($action->execute($request->validated()));
    }

    public function replace(
        int $entityId,
        CreateOrReplaceCategoryRequest $request,
        ReplaceCategoryAction $action
    ): CategoriesResource {
        return new CategoriesResource($action->execute($entityId, $request->validated()));
    }

    public function patch(int $entityId, PatchCategoryRequest $request, PatchCategoryAction $action): CategoriesResource
    {
        return new CategoriesResource($action->execute($entityId, $request->validated()));
    }

    public function delete(int $entityId, DeleteCategoryAction $action): EmptyResource
    {
        $action->execute($entityId);
        return new EmptyResource();
    }

    public function massDelete(MassDeleteRequest $request, DeleteManyCategoriesAction $action): EmptyResource
    {
        $action->execute($request->getIds());
        return new EmptyResource();
    }

    public function uploadImage(
        int $entityId,
        UploadCategoryImageRequest $request,
        SaveCategoryImageAction $action
    ): CategoriesResource {
        return new CategoriesResource($action->execute($entityId, $request->file('file')));
    }

    public function deleteImage(int $entityId, DeleteCategoryImageAction $action): CategoriesResource
    {
        return new CategoriesResource($action->execute($entityId));
    }

    public function properties(
        int $entityId,
        Request $request,
        CategoryPropertiesQueryFactory $factory
    ): AnonymousResourceCollection {
        return CategoryPropertiesResource::collection(
            $factory->createQuery($entityId, $request)->get()
        );
    }

    public function tree(CategoriesTreeQuery $query): CategoriesTreeResourceCollection
    {
        return new CategoriesTreeResourceCollection($query->get());
    }
}
