<?php

namespace App\Http\ApiV1\Modules\Classifiers\Controllers;

use App\Domain\Classifiers\Actions\Brands\CreateBrandAction;
use App\Domain\Classifiers\Actions\Brands\DeleteBrandAction;
use App\Domain\Classifiers\Actions\Brands\DeleteBrandImageAction;
use App\Domain\Classifiers\Actions\Brands\DeleteManyBrandsAction;
use App\Domain\Classifiers\Actions\Brands\PatchBrandAction;
use App\Domain\Classifiers\Actions\Brands\ReplaceBrandAction;
use App\Domain\Classifiers\Actions\Brands\SaveBrandImageAction;
use App\Domain\Classifiers\Models\Brand;
use App\Http\ApiV1\Modules\Classifiers\Queries\BrandsQuery;
use App\Http\ApiV1\Modules\Classifiers\Requests\CreateOrReplaceBrandRequest;
use App\Http\ApiV1\Modules\Classifiers\Requests\PatchBrandRequest;
use App\Http\ApiV1\Modules\Classifiers\Requests\UploadBrandImageRequest;
use App\Http\ApiV1\Modules\Classifiers\Resources\BrandsResource;
use App\Http\ApiV1\Support\Pagination\PageBuilderFactory;
use App\Http\ApiV1\Support\Requests\MassDeleteRequest;
use App\Http\ApiV1\Support\Resources\EmptyResource;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;

class BrandsController
{
    public function search(PageBuilderFactory $pageBuilderFactory, BrandsQuery $query): AnonymousResourceCollection
    {
        return BrandsResource::collectPage(
            $pageBuilderFactory->fromQuery($query)->build()
        );
    }

    public function get(int $entityId, BrandsQuery $query): BrandsResource
    {
        return new BrandsResource($query->findOrFail($entityId));
    }

    public function create(CreateOrReplaceBrandRequest $request, CreateBrandAction $action): BrandsResource
    {
        return new BrandsResource($action->execute($request->validated()));
    }

    public function replace(
        int $entityId,
        CreateOrReplaceBrandRequest $request,
        ReplaceBrandAction $action
    ): BrandsResource {
        return new BrandsResource($action->execute($entityId, $request->validated()));
    }

    public function patch(int $entityId, PatchBrandRequest $request, PatchBrandAction $action): BrandsResource
    {
        return new BrandsResource($action->execute($entityId, $request->validated()));
    }

    public function delete(int $entityId, DeleteBrandAction $action): EmptyResource
    {
        $action->execute($entityId);
        return new EmptyResource();
    }

    public function massDelete(MassDeleteRequest $request, DeleteManyBrandsAction $action): EmptyResource
    {
        $action->execute($request->getIds());
        return new EmptyResource();
    }

    public function uploadImage(int $entityId, UploadBrandImageRequest $request, SaveBrandImageAction $action): BrandsResource
    {
        return new BrandsResource($action->execute($entityId, $request->file('file')));
    }

    public function deleteImage(int $entityId, DeleteBrandImageAction $action): BrandsResource
    {
        return new BrandsResource($action->execute($entityId));
    }
}
