<?php

namespace App\Http\ApiV1\Modules\Classifiers\Controllers;

use App\Domain\Classifiers\Actions\Countries\CreateCountryAction;
use App\Domain\Classifiers\Actions\Countries\DeleteCountryAction;
use App\Domain\Classifiers\Actions\Countries\DeleteManyCountriesAction;
use App\Domain\Classifiers\Actions\Countries\PatchCountryAction;
use App\Domain\Classifiers\Actions\Countries\ReplaceCountryAction;
use App\Domain\Classifiers\Models\Country;
use App\Http\ApiV1\Modules\Classifiers\Queries\CountriesQuery;
use App\Http\ApiV1\Modules\Classifiers\Requests\CreateOrReplaceCountryRequest;
use App\Http\ApiV1\Modules\Classifiers\Requests\PatchCountryRequest;
use App\Http\ApiV1\Modules\Classifiers\Resources\CountriesResource;
use App\Http\ApiV1\Support\Pagination\PageBuilderFactory;
use App\Http\ApiV1\Support\Requests\MassDeleteRequest;
use App\Http\ApiV1\Support\Resources\EmptyResource;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;

class CountriesController
{
    public function search(PageBuilderFactory $pageBuilderFactory, CountriesQuery $query): AnonymousResourceCollection
    {
        return CountriesResource::collectPage(
            $pageBuilderFactory->fromQuery($query)->build()
        );
    }

    public function get(int $entityId, CountriesQuery $query): CountriesResource
    {
        return new CountriesResource($query->findOrFail($entityId));
    }

    public function create(CreateOrReplaceCountryRequest $request, CreateCountryAction $action): CountriesResource
    {
        return new CountriesResource($action->execute($request->validated()));
    }

    public function replace(
        int $entityId,
        CreateOrReplaceCountryRequest $request,
        ReplaceCountryAction $action
    ): CountriesResource {
        return new CountriesResource($action->execute($entityId, $request->validated()));
    }

    public function patch(int $entityId, PatchCountryRequest $request, PatchCountryAction $action): CountriesResource
    {
        return new CountriesResource($action->execute($entityId, $request->validated()));
    }

    public function delete(int $entityId, DeleteCountryAction $action): EmptyResource
    {
        $action->execute($entityId);
        return new EmptyResource();
    }

    public function massDelete(MassDeleteRequest $request, DeleteManyCountriesAction $action): EmptyResource
    {
        $action->execute($request->getIds());
        return new EmptyResource();
    }
}
