<?php

namespace App\Http\ApiV1\Modules\Classifiers\Controllers;

use App\Domain\Classifiers\Actions\Manufacturers\CreateManufacturerAction;
use App\Domain\Classifiers\Actions\Manufacturers\DeleteManufacturerAction;
use App\Domain\Classifiers\Actions\Manufacturers\DeleteManyManufacturersAction;
use App\Domain\Classifiers\Actions\Manufacturers\PatchManufacturerAction;
use App\Domain\Classifiers\Actions\Manufacturers\ReplaceManufacturerAction;
use App\Domain\Classifiers\Models\Manufacturer;
use App\Http\ApiV1\Modules\Classifiers\Queries\ManufacturersQuery;
use App\Http\ApiV1\Modules\Classifiers\Requests\CreateOrReplaceManufacturerRequest;
use App\Http\ApiV1\Modules\Classifiers\Requests\PatchManufacturerRequest;
use App\Http\ApiV1\Modules\Classifiers\Resources\ManufacturersResource;
use App\Http\ApiV1\Support\Pagination\PageBuilderFactory;
use App\Http\ApiV1\Support\Requests\MassDeleteRequest;
use App\Http\ApiV1\Support\Resources\EmptyResource;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;

class ManufacturersController
{
    public function search(PageBuilderFactory $pageBuilderFactory, ManufacturersQuery $query): AnonymousResourceCollection
    {
        return ManufacturersResource::collectPage(
            $pageBuilderFactory->fromQuery($query)->build()
        );
    }

    public function get(int $entityId, ManufacturersQuery $query): ManufacturersResource
    {
        return new ManufacturersResource($query->findOrFail($entityId));
    }

    public function create(
        CreateOrReplaceManufacturerRequest $request,
        CreateManufacturerAction $action
    ): ManufacturersResource {
        return new ManufacturersResource($action->execute($request->validated()));
    }

    public function replace(
        int $entityId,
        CreateOrReplaceManufacturerRequest $request,
        ReplaceManufacturerAction $action
    ): ManufacturersResource {
        return new ManufacturersResource($action->execute($entityId, $request->validated()));
    }

    public function patch(
        int $entityId,
        PatchManufacturerRequest $request,
        PatchManufacturerAction $action
    ): ManufacturersResource {
        return new ManufacturersResource($action->execute($entityId, $request->validated()));
    }

    public function delete(int $entityId, DeleteManufacturerAction $action): EmptyResource
    {
        $action->execute($entityId);
        return new EmptyResource();
    }

    public function massDelete(MassDeleteRequest $request, DeleteManyManufacturersAction $action): EmptyResource
    {
        $action->execute($request->getIds());
        return new EmptyResource();
    }
}
