<?php

namespace App\Http\ApiV1\Modules\Classifiers\Controllers;

use App\Domain\Classifiers\Actions\ProductTypes\CreateProductTypeAction;
use App\Domain\Classifiers\Actions\ProductTypes\DeleteManyProductTypesAction;
use App\Domain\Classifiers\Actions\ProductTypes\DeleteProductTypeAction;
use App\Domain\Classifiers\Actions\ProductTypes\PatchProductTypeAction;
use App\Domain\Classifiers\Actions\ProductTypes\ReplaceProductTypeAction;
use App\Domain\Classifiers\Models\ProductType;
use App\Http\ApiV1\Modules\Classifiers\Queries\ProductTypesQuery;
use App\Http\ApiV1\Modules\Classifiers\Requests\CreateOrReplaceProductTypeRequest;
use App\Http\ApiV1\Modules\Classifiers\Requests\PatchProductTypeRequest;
use App\Http\ApiV1\Modules\Classifiers\Resources\ProductTypesResource;
use App\Http\ApiV1\Support\Pagination\PageBuilderFactory;
use App\Http\ApiV1\Support\Requests\MassDeleteRequest;
use App\Http\ApiV1\Support\Resources\EmptyResource;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;

class ProductTypesController
{
    public function search(PageBuilderFactory $pageBuilderFactory, ProductTypesQuery $query): AnonymousResourceCollection
    {
        return ProductTypesResource::collectPage(
            $pageBuilderFactory->fromQuery($query)->build()
        );
    }

    public function get(int $entityId, ProductTypesQuery $query): ProductTypesResource
    {
        return new ProductTypesResource($query->findOrFail($entityId));
    }

    public function create(
        CreateOrReplaceProductTypeRequest $request,
        CreateProductTypeAction $action
    ): ProductTypesResource {
        return new ProductTypesResource($action->execute($request->validated()));
    }

    public function replace(
        int $entityId,
        CreateOrReplaceProductTypeRequest $request,
        ReplaceProductTypeAction $action
    ): ProductTypesResource {
        return new ProductTypesResource($action->execute($entityId, $request->validated()));
    }

    public function patch(
        int $entityId,
        PatchProductTypeRequest $request,
        PatchProductTypeAction $action
    ): ProductTypesResource {
        return new ProductTypesResource($action->execute($entityId, $request->validated()));
    }

    public function delete(int $entityId, DeleteProductTypeAction $action): EmptyResource
    {
        $action->execute($entityId);
        return new EmptyResource();
    }

    public function massDelete(MassDeleteRequest $request, DeleteManyProductTypesAction $action): EmptyResource
    {
        $action->execute($request->getIds());
        return new EmptyResource();
    }
}
