<?php

namespace App\Http\ApiV1\Modules\Products\Controllers;

use App\Domain\Products\Actions\Images\CreateProductImageAction;
use App\Domain\Products\Actions\Images\DeleteProductImageAction;
use App\Domain\Products\Actions\Images\PatchProductImageAction;
use App\Domain\Products\Actions\Images\UploadProductImageAction;
use App\Http\ApiV1\Modules\Products\Queries\ProductImagesQuery;
use App\Http\ApiV1\Modules\Products\Requests\CreateProductImageRequest;
use App\Http\ApiV1\Modules\Products\Requests\PatchProductImageRequest;
use App\Http\ApiV1\Modules\Products\Requests\UploadProductImageRequest;
use App\Http\ApiV1\Modules\Products\Resources\ProductImagesResource;
use App\Http\ApiV1\Support\Pagination\PageBuilderFactory;
use App\Http\ApiV1\Support\Resources\EmptyResource;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;

class ProductImagesController
{
    public function create(
        CreateProductImageRequest $request,
        CreateProductImageAction $action
    ): ProductImagesResource {
        return new ProductImagesResource($action->execute($request->validated()));
    }

    public function upload(
        int $imageId,
        UploadProductImageRequest $request,
        UploadProductImageAction $action
    ): ProductImagesResource {
        return new ProductImagesResource($action->execute($imageId, $request->file('file')));
    }

    public function patch(
        int $entityId,
        PatchProductImageRequest $request,
        PatchProductImageAction $action
    ): ProductImagesResource {
        return new ProductImagesResource($action->execute($entityId, $request->validated()));
    }

    public function delete(int $entityId, DeleteProductImageAction $action): EmptyResource
    {
        $action->execute($entityId);
        return new EmptyResource();
    }

    public function get(int $entityId, ProductImagesQuery $query): ProductImagesResource
    {
        return new ProductImagesResource($query->findOrFail($entityId));
    }

    public function search(PageBuilderFactory $pageBuilderFactory, ProductImagesQuery $query): AnonymousResourceCollection
    {
        return ProductImagesResource::collectPage(
            $pageBuilderFactory->fromQuery($query)->build()
        );
    }
}
