<?php

namespace App\Http\ApiV1\Modules\Products\Queries;

use App\Domain\Products\Models\Product;
use Spatie\QueryBuilder\AllowedFilter;
use Spatie\QueryBuilder\AllowedInclude;
use Spatie\QueryBuilder\QueryBuilder;

class ProductsQuery extends QueryBuilder
{
    public function __construct()
    {
        parent::__construct(Product::query());

        $this->allowedSorts(['id', 'name', 'code']);
        $this->defaultSort('id');

        $this->allowedFilters([
            AllowedFilter::exact('id'),
            AllowedFilter::exact('code'),
            AllowedFilter::partial('name'),
            AllowedFilter::scope('category_id', 'in_categories'),
            AllowedFilter::exact('brand_id'),
            AllowedFilter::exact('product_type_id'),
            AllowedFilter::exact('archive'),
            AllowedFilter::exact('external_id'),
        ]);

        $this->allowedIncludes([
            'brand',
            'category',
            'manufacturer',
            'country',
            'images',
            AllowedInclude::relationship('type', 'productType'),
            AllowedInclude::custom('attributes', new PropertyDirectoryValueInclude())
        ]);
    }
}
