<?php

namespace App\Http\ApiV1\Modules\Products\Requests;

use App\Domain\Categories\Models\Category;
use App\Domain\Classifiers\Models\Brand;
use App\Domain\Classifiers\Models\Country;
use App\Domain\Classifiers\Models\Manufacturer;
use App\Domain\Classifiers\Models\ProductType;
use App\Domain\Products\Models\ProductArchiveType;
use App\Http\ApiV1\Support\Requests\BaseFormRequest;
use BenSampo\Enum\Rules\EnumValue;
use Illuminate\Validation\Rule;

class CreateOrReplaceProductRequest extends BaseFormRequest
{
    public function rules(): array
    {
        return [
            'external_id' => ['required', 'string'],
            'name' => ['required', 'string'],
            'code' => ['required', 'string'],
            'description' => ['nullable', 'string'],
            'product_type_id' => ['nullable', Rule::exists(ProductType::class, 'id')],
            'brand_id' => ['nullable', Rule::exists(Brand::class, 'id')],
            'category_id' => ['nullable', Rule::exists(Category::class, 'id')],
            'manufacturer_id' => ['nullable', Rule::exists(Manufacturer::class, 'id')],
            'country_id' => ['nullable', Rule::exists(Country::class, 'id')],
            'barcode' => ['nullable', 'string'],
            'weight' => ['nullable', 'numeric'],
            'weight_gross' => ['nullable', 'numeric'],
            'length' => ['nullable', 'numeric'],
            'width' => ['nullable', 'numeric'],
            'height' => ['nullable', 'numeric'],
            'ingredients' => ['nullable', 'string'],
            'is_new' => ['nullable', 'boolean'],
            'archive' => ['nullable', new EnumValue(ProductArchiveType::class)],
        ];
    }
}
