<?php

namespace App\Http\ApiV1\Modules\Products\Requests;

use App\Domain\Categories\Models\Property;
use App\Http\ApiV1\Support\Requests\BaseFormRequest;
use Illuminate\Support\Collection;
use Illuminate\Validation\Rule;

class ReplaceProductAttributesRequest extends BaseFormRequest
{
    public function rules(): array
    {
        return [
            'values' => ['nullable', 'array'],
            'values.*.property_id' => ['required', Rule::exists(Property::class, 'id')],
            'values.*.value' => ['present'],
        ];
    }

    public function values(): array
    {
        return collect($this->validated()['values'] ?? [])
            ->groupBy('property_id')
            ->map(fn(Collection $items) => $this->mergeValues($items))
            ->toArray();
    }

    protected function mergeValues(Collection $values)
    {
        return $values->reduce(function (array $carry, mixed $item) {
            $value = array_filter((array)($item['value'] ?? []));
            return array_merge($carry, $value);
        }, []);
    }
}
