<?php

namespace App\Http\ApiV1\Modules\Products\Resources;

use App\Domain\Categories\Models\PropertyType;
use App\Domain\Products\Models\ProductPropertyValue;
use App\Http\ApiV1\Modules\Categories\Resources\AttributeDirectoryValuesResource;
use Illuminate\Http\Resources\Json\ResourceCollection;
use Illuminate\Support\Collection;

/**
 * @property Collection|ProductPropertyValue[]
 */
class ProductAttributesCollection extends ResourceCollection
{
    public function toArray($request): array
    {
        return $this->collection
            ->groupBy('property_id')
            ->map(fn(Collection $values) => $this->mergeValues($values))
            ->toArray();
    }

    private function mergeValues(Collection $productValues): array
    {
        /** @var ProductPropertyValue $template */
        $template = $productValues->first();
        $values = $productValues->map(fn(ProductPropertyValue $item) => $item->value());

        $result = [
            'property_id' => $template->property_id,
            'type' => $template->type,
            'value' => $values
        ];

        if ($template->type->is(PropertyType::DIRECTORY())) {
            $result['directory'] = AttributeDirectoryValuesResource::collection(
                $productValues->map(fn(ProductPropertyValue $item) => $item->propertyDirectoryValue)
            );
        }

        return $result;
    }
}
