<?php

namespace App\Http\ApiV1\Modules\Products\Resources;

use App\Domain\Products\Models\Product;
use App\Http\ApiV1\Modules\Categories\Resources\CategoriesResource;
use App\Http\ApiV1\Modules\Classifiers\Resources\BrandsResource;
use App\Http\ApiV1\Modules\Classifiers\Resources\CountriesResource;
use App\Http\ApiV1\Modules\Classifiers\Resources\ManufacturersResource;
use App\Http\ApiV1\Modules\Classifiers\Resources\ProductTypesResource;
use App\Http\ApiV1\Support\Resources\BaseJsonResource;

/**
 * @mixin Product
 */
class ProductsResource extends BaseJsonResource
{
    public function toArray($request): array
    {
        return [
            'id' => $this->id,
            'code' => $this->code,
            'name' => $this->name,
            'description' => $this->description,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,

            'external_id' => $this->external_id,
            'product_type_id' => $this->product_type_id,
            'country_id' => $this->country_id,
            'manufacturer_id' => $this->manufacturer_id,
            'brand_id' => $this->brand_id,
            'category_id' => $this->category_id,
            'variant_group_id' => $this->variant_group_id,

            'barcode' => $this->barcode,
            'weight' => $this->weight,
            'weight_gross' => $this->weight_gross,
            'length' => $this->length,
            'height' => $this->height,
            'width' => $this->width,
            'ingredients' => $this->ingredients,
            'storage_area' => $this->storage_area,

            'archive' => $this->archive,
            'is_new' => $this->is_new,

            'brand' => $this->whenLoaded('brand', function () {
                return new BrandsResource($this->brand);
            }),

            'type' => $this->whenLoaded('productType', function () {
                return new ProductTypesResource($this->productType);
            }),

            'category' => $this->whenLoaded('category', function () {
                return new CategoriesResource($this->category);
            }),

            'country' => $this->whenLoaded('country', function () {
                return new CountriesResource($this->country);
            }),

            'manufacturer' => $this->whenLoaded('manufacturer', function () {
                return new ManufacturersResource($this->manufacturer);
            }),

            'images' => $this->whenLoaded('images', function () {
                return ProductImagesResource::collection($this->images);
            }),

            'attributes' => $this->whenLoaded(
                'properties',
                fn() => new ProductAttributesCollection($this->properties)
            )
        ];
    }
}
