<?php

use App\Http\ApiV1\Modules\Products\Controllers\ProductImagesController;
use App\Http\ApiV1\Modules\Products\Controllers\ProductsController;
use App\Http\ApiV1\Modules\Products\Controllers\VariantGroupsController;
use Illuminate\Support\Facades\Route;

//region Products
Route::post('products', [ProductsController::class, 'create'])->name('createProduct');

Route::prefix('products/{entityId}')
    ->group(function () {
        Route::get('', [ProductsController::class, 'get'])->name('getProduct');
        Route::put('', [ProductsController::class, 'replace'])->name('replaceProduct');
        Route::patch('', [ProductsController::class, 'patch'])->name('patchProduct');
        Route::delete('', [ProductsController::class, 'delete'])->name('deleteProduct');
    });

Route::post('products:search', [ProductsController::class, 'search'])
    ->name('searchProducts');

Route::post('products:mass-delete', [ProductsController::class, 'massDelete'])
    ->name('deleteProducts');

Route::put('products/{entityId}:attributes', [ProductsController::class, 'replaceAttributes'])
    ->name('replaceProductAttributes');

Route::patch('products/{entityId}:attributes', [ProductsController::class, 'patchAttributes'])
    ->name('patchProductAttributes');

Route::post('products:mark-for-index', [ProductsController::class, 'markForIndex'])
    ->name('markForIndexProducts');

//endregion

//region Product images
Route::post('images', [ProductImagesController::class, 'create'])
    ->name('createProductImage');

Route::prefix('images/{entityId}')
    ->group(function () {
        Route::get('', [ProductImagesController::class, 'get'])->name('getProductImage');
        Route::patch('', [ProductImagesController::class, 'patch'])->name('patchProductImage');
        Route::delete('', [ProductImagesController::class, 'delete'])->name('deleteProductImage');
    });

Route::post('images/{entityId}:upload-image', [ProductImagesController::class, 'upload'])
    ->name('uploadProductImage');

Route::post('images:search', [ProductImagesController::class, 'search'])
    ->name('searchProductImages');
//endregion

//region Variants
Route::post('variants', [VariantGroupsController::class, 'create'])->name('createVariantGroup');

Route::prefix('variants/{entityId}')
    ->group(function () {
        Route::get('', [VariantGroupsController::class, 'get'])->name('getVariantGroup');
        Route::put('', [VariantGroupsController::class, 'replace'])->name('replaceVariantGroup');
        Route::patch('', [VariantGroupsController::class, 'patch'])->name('patchVariantGroup');
        Route::delete('', [VariantGroupsController::class, 'delete'])->name('deleteVariantGroup');
    });

Route::post('variants:search', [VariantGroupsController::class, 'search'])
    ->name('searchVariantGroups');
//endregion
