<?php

namespace App\Observers;

use App\Core\Elastic\IndexFactory;
use App\Core\Elastic\ScheduleRunner;
use App\Domain\Products\Models\Product;
use App\Domain\Products\Models\ProductArchiveType;

/**
 * Class ProductObserver
 * @package App\Observers
 */
class ProductObserver
{
    /**
     * Handle the product "creating" event.
     * @param  Product  $product
     * @return void
     */
    public function creating(Product $product)
    {
        if (is_null($product->archive)) {
            $product->archive = ProductArchiveType::NO_ARCHIVE();
        }
    }

    /**
     * Handle the product "created" event.
     * @param  Product  $product
     * @return void
     */
    public function created(Product $product)
    {
//        History::saveEvent(HistoryType::TYPE_CREATE, $product, $product);
        $this->scheduleIndex($product->id);
    }

    /**
     * Handle the product "updated" event.
     * @param  Product  $product
     * @return void
     */
    public function updated(Product $product)
    {
//        History::saveEvent(HistoryType::TYPE_UPDATE, $product, $product);
        $this->scheduleIndex($product->id);
    }

    /**
     * @param Product $product
     */
    public function saving(Product $product)
    {
        $this->setArchiveAt($product);
    }

    /**
     * Handle the product "deleting" event.
     *
     * @param  Product  $product
     * @throws \Exception
     */
    public function deleting(Product $product)
    {
//        History::saveEvent(HistoryType::TYPE_DELETE, $product, $product);
    }

    /**
     * @param Product $product
     */
    public function deleted(Product $product)
    {
        $this->scheduleIndex($product->id);
    }

    /**
     * @param int $id
     */
    private function scheduleIndex(int $id)
    {
        $scheduler = new ScheduleRunner();
        $scheduler->markForIndex(IndexFactory::TYPE_PRODUCT, $id);
    }

    private function setArchiveAt(Product $product)
    {
        if ($product->archive != $product->getOriginal('archive')) {
            $product->archive_date = now();
        }
    }
}
