<?php

namespace App\Observers;

use App\Core\Elastic\IndexFactory;
use App\Core\Elastic\ScheduleRunner;
use App\Domain\Products\Models\ProductPropertyValue;

class ProductPropertyValueObserver
{
    protected $scheduler;

    public function __construct(ScheduleRunner $scheduler)
    {
        $this->scheduler = $scheduler;
    }

    /**
     * Handle the model "created" event.
     *
     * @param  \App\Domain\Products\Models\ProductPropertyValue  $value
     * @return void
     */
    public function created(ProductPropertyValue $value)
    {
        $this->scheduler->markForIndex(IndexFactory::TYPE_PRODUCT, $value->product_id);
    }

    /**
     * Handle the model "updated" event.
     *
     * @param  \App\Domain\Products\Models\ProductPropertyValue  $value
     * @return void
     */
    public function updated(ProductPropertyValue $value)
    {
        $this->scheduler->markForIndex(IndexFactory::TYPE_PRODUCT, $value->product_id);
    }

    /**
     * Handle the model "deleted" event.
     *
     * @param  \App\Domain\Products\Models\ProductPropertyValue  $value
     * @return void
     */
    public function deleted(ProductPropertyValue $value)
    {
        $this->scheduler->markForIndex(IndexFactory::TYPE_PRODUCT, $value->product_id);
    }
}
