<?php

namespace Database\Factories\Categories;

use App\Domain\Categories\Models\Category;
use Database\Factories\Factory;
use Illuminate\Support\Collection;

/**
 * @method Category createOne(array $fields = [])
 * @method Category makeOne(array $fields = [])
 * @method Category|Category[]|Collection create(array $fields = [], $parent = null)
 */
class CategoriesFactory extends Factory
{
    protected $model = Category::class;

    public function definition(): array
    {
        return [
            'name' => $this->faker->word,
            'code' => $this->faker->slug(2),
        ];
    }

    public function withParent(Category $category): self
    {
        return $this->state(function () use ($category) {
            return [
                'parent_id' => $category->getKey(),
            ];
        });
    }

    public function createTree(int $depth = 2): Collection
    {
        $results = new Collection();
        $parent = null;

        for($i = 0; $i < $depth; $i++) {
            $fields = [];
            if ($parent !== null) {
                $fields['parent_id'] = $parent->id;
            }

            $parent = $this->createOne($fields);
            $results->push($parent);
        }

        return $results;
    }
}
