<?php

namespace Database\Factories\Products;

use App\Domain\Categories\Models\Category;
use App\Domain\Classifiers\Models\Brand;
use App\Domain\Classifiers\Models\Country;
use App\Domain\Classifiers\Models\Manufacturer;
use App\Domain\Classifiers\Models\ProductType;
use App\Domain\Products\Models\Product;
use App\Domain\Products\Models\ProductArchiveType;
use Database\Factories\Factory;
use Illuminate\Support\Collection;

/**
 * @method Product createOne(array $fields = [])
 * @method Product makeOne(array $fields = [])
 * @method Product|Product[]|Collection create(array $fields = [], $parent = null)
 */
class ProductsFactory extends Factory
{
    protected $model = Product::class;

    public function definition(): array
    {
        return [
            'name' => $this->faker->sentence(),
            'code' => $this->faker->unique()->slug,
            'description' => $this->faker->text,

            'external_id' => $this->faker->unique()->numerify('######'),
            'barcode' => $this->faker->ean13,
            'weight' => $this->faker->randomFloat(3, 1, 100),
            'weight_gross' => $this->faker->randomFloat(3, 1, 100),
            'length' => $this->faker->randomFloat(2, 1, 1000),
            'height' => $this->faker->randomFloat(2, 1, 1000),
            'width' => $this->faker->randomFloat(2, 1, 1000),

            'archive' => ProductArchiveType::NO_ARCHIVE,
            'is_new' => $this->faker->boolean,
        ];
    }

    public function inCategory(Category $category): self
    {
        return $this->state(fn() => ['category_id' => $category->id]);
    }

    public function withRelations(
        ?Brand $brand = null,
        ?ProductType $productType = null,
        ?Manufacturer $manufacturer = null,
        ?Country $country = null
    ): self {
        $fields = array_filter([
            'brand_id' => $brand?->id,
            'product_type_id' => $productType?->id,
            'manufacturer_id' => $manufacturer?->id,
            'country_id' => $country?->id,
        ]);

        return $this->state($fields);
    }
}
