<?php

namespace Tests\Feature\Categories;

use App\Domain\Categories\Models\Property;
use App\Domain\Categories\Models\PropertyDirectoryValue;
use App\Domain\Categories\Models\PropertyType;
use Tests\APITestCase;

class DirectoryValuesTest extends APITestCase
{
    private static $jsonStructure = [
        'id',
        'code',
        'name',
        'property_id',
    ];

    /** @var Property */
    private $property;

    /** @var PropertyDirectoryValue */
    private $value;

    protected function setUp(): void
    {
        parent::setUp();

        $this->property = Property::factory()->createOne(['type' => PropertyType::DIRECTORY]);
        $this->value = PropertyDirectoryValue::factory()->forProperty($this->property)->createOne();
    }

    public function testCreate(): void
    {
        $data = PropertyDirectoryValue::factory()->makeFillableData();

        $response = $this->apiCreate('createDirectoryValue', $data, $this->property);

        $response->assertStatus(201);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
        $response->assertJsonFragment($data);
    }

    public function testReplace(): void
    {
        $data = PropertyDirectoryValue::factory()->makeFillableData();

        $response = $this->apiReplace('replaceDirectoryValue', $this->value, $data);

        $response->assertStatus(200);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
        $response->assertJsonFragment($data);
    }

    public function testPatch(): void
    {
        $data = ['name' => 'New patch name'];

        $response = $this->apiPatch('patchDirectoryValue', $this->value, $data);

        $response->assertStatus(200);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
        $response->assertJsonFragment($data);
    }

    public function testDelete(): void
    {
        $response = $this->apiDelete('deleteDirectoryValue', $this->value);

        $response->assertStatus(200);
        $response->assertJsonFragment(['data' => null]);
        $this->assertNull(PropertyDirectoryValue::find($this->value->id));
    }

    public function testGet(): void
    {
        $response = $this->apiGet('getDirectoryValue', $this->value);

        $response->assertStatus(200);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
    }

    public function testSearch(): void
    {
        $response = $this->apiSearch('searchDirectoryValues', ['code' => $this->value->code]);

        $response->assertStatus(200);
        $response->assertJsonCount(1, 'data');
        $response->assertJsonFragment(['name' => $this->value->name]);
    }
}
