<?php

namespace Tests\Feature\Classifiers;

use App\Domain\Classifiers\Models\Brand;
use Tests\APITestCase;

class BrandsTest extends APITestCase
{
    private static $jsonStructure = [
        'id',
        'code',
        'name',
        'description',
        'file',
    ];

    public function testCreate(): void
    {
        $data = Brand::factory()->makeFillableData();

        $response = $this->apiCreate('createBrand', $data);

        $response->assertStatus(201);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
        $response->assertJsonFragment($data);
    }

    public function testReplace(): void
    {
        $brand = Brand::factory()->createOne();
        $data = Brand::factory()->makeFillableData();

        $response = $this->apiReplace('replaceBrand', $brand, $data);

        $response->assertStatus(200);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
        $response->assertJsonFragment($data);
    }

    public function testPatch(): void
    {
        $brand = Brand::factory()->createOne();
        $data = ['name' => 'New patch name'];

        $response = $this->apiPatch('patchBrand', $brand, $data);

        $response->assertStatus(200);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
        $response->assertJsonFragment($data);
    }

    public function testDelete(): void
    {
        $brand = Brand::factory()->createOne();

        $response = $this->apiDelete('deleteBrand', $brand);

        $response->assertStatus(200);
        $response->assertJsonFragment(['data' => null]);
        $this->assertNull(Brand::find($brand->id));
    }

    public function testMassDelete(): void
    {
        $ids = Brand::factory()
            ->count(3)
            ->create()
            ->pluck('id')
            ->toArray();

        $response = $this->apiMassDelete('deleteBrands', $ids);

        $response->assertStatus(200);
        $response->assertJsonFragment(['data' => null]);

        $this->assertCount(0, Brand::find($ids));
    }

    public function testGet(): void
    {
        $brand = Brand::factory()->createOne();

        $response = $this->apiGet('getBrand', $brand);

        $response->assertStatus(200);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
    }

    public function testSearch(): void
    {
        $brand = Brand::factory()->count(3)->create()[1];

        $response = $this->apiSearch('searchBrands', ['code' => $brand->code]);

        $response->assertStatus(200);
        $response->assertJsonCount(1, 'data');
        $response->assertJsonFragment(['name' => $brand->name]);
    }
}
