<?php

namespace Tests\Feature\Classifiers;

use App\Domain\Classifiers\Models\Country;
use Tests\APITestCase;

class CountriesTest extends APITestCase
{
    private static $jsonStructure = ['id', 'code', 'name'];

    public function testCreate(): void
    {
        $data = Country::factory()->makeFillableData();

        $response = $this->apiCreate('createCountry', $data);

        $response->assertStatus(201);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
        $response->assertJsonFragment($data);
    }

    public function testReplace(): void
    {
        $country = Country::factory()->createOne();
        $data = Country::factory()->makeFillableData();

        $response = $this->apiReplace('replaceCountry', $country, $data);

        $response->assertStatus(200);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
        $response->assertJsonFragment($data);
    }

    public function testPatch(): void
    {
        $country = Country::factory()->createOne();
        $data = ['name' => 'New patch name'];

        $response = $this->apiPatch('patchCountry', $country, $data);

        $response->assertStatus(200);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
        $response->assertJsonFragment($data);
    }

    public function testDelete(): void
    {
        $country = Country::factory()->createOne();

        $response = $this->apiDelete('deleteCountry', $country);

        $response->assertStatus(200);
        $response->assertJsonFragment(['data' => null]);
        $this->assertNull(Country::find($country->id));
    }

    public function testMassDelete(): void
    {
        $ids = Country::factory()
            ->count(3)
            ->create()
            ->pluck('id')
            ->toArray();

        $response = $this->apiMassDelete('deleteCountries', $ids);

        $response->assertStatus(200);
        $response->assertJsonFragment(['data' => null]);

        $this->assertCount(0, Country::find($ids));
    }

    public function testGet(): void
    {
        $country = Country::factory()->createOne();

        $response = $this->apiGet('getCountry', $country);

        $response->assertStatus(200);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
    }

    public function testSearch(): void
    {
        $country = Country::factory()->count(3)->create()[1];

        $response = $this->apiSearch('searchCountries', ['code' => $country->code]);

        $response->assertStatus(200);
        $response->assertJsonCount(1, 'data');
        $response->assertJsonFragment(['name' => $country->name]);
    }
}
