<?php

namespace Tests\Feature\Classifiers;

use App\Domain\Classifiers\Models\Manufacturer;
use Illuminate\Foundation\Testing\DatabaseTransactions;
use Tests\APITestCase;

class ManufacturersTest extends APITestCase
{
    use DatabaseTransactions;

    private static $jsonStructure = ['id', 'code', 'name'];

    public function testCreate(): void
    {
        $data = Manufacturer::factory()->makeFillableData();

        $response = $this->apiCreate('createManufacturer', $data);

        $response->assertStatus(201);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
    }

    public function testReplace(): void
    {
        $manufacturer = Manufacturer::factory()->createOne();
        $data = Manufacturer::factory()->makeFillableData();

        $response = $this->apiReplace('replaceManufacturer', $manufacturer, $data);

        $response->assertStatus(200);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
        $response->assertJsonFragment($data);
    }

    public function testPatch(): void
    {
        $manufacturer = Manufacturer::factory()->createOne();
        $data = ['name' => 'New patch name'];

        $response = $this->apiPatch('patchManufacturer', $manufacturer, $data);

        $response->assertStatus(200);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
        $response->assertJsonFragment($data);
    }

    public function testDelete(): void
    {
        $manufacturer = Manufacturer::factory()->createOne();

        $response = $this->apiDelete('deleteManufacturer', $manufacturer);

        $response->assertStatus(200);
        $response->assertJsonFragment(['data' => null]);
        $this->assertNull(Manufacturer::find($manufacturer->id));
    }

    public function testMassDelete(): void
    {
        $ids = Manufacturer::factory()
            ->count(3)
            ->create()
            ->pluck('id')
            ->toArray();

        $response = $this->apiMassDelete('deleteManufacturers', $ids);

        $response->assertStatus(200);
        $response->assertJsonFragment(['data' => null]);

        $this->assertCount(0, Manufacturer::find($ids));
    }

    public function testGet(): void
    {
        $manufacturer = Manufacturer::factory()->createOne();

        $response = $this->apiGet('getManufacturer', $manufacturer);

        $response->assertStatus(200);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
    }

    public function testSearch(): void
    {
        $manufacturer = Manufacturer::factory()->count(3)->create()[1];

        $response = $this->apiSearch('searchManufacturers', ['code' => $manufacturer->code]);

        $response->assertStatus(200);
        $response->assertJsonCount(1, 'data');
        $response->assertJsonFragment(['name' => $manufacturer->name]);
    }
}
