<?php

namespace Tests\Feature\Classifiers;

use App\Domain\Classifiers\Models\ProductType;
use Tests\APITestCase;

class ProductTypesTest extends APITestCase
{
    private static $jsonStructure = ['id', 'code', 'name'];

    public function testCreate(): void
    {
        $data = ProductType::factory()->makeFillableData();

        $response = $this->apiCreate('createProductType', $data);

        $response->assertStatus(201);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
        $response->assertJsonFragment($data);
    }

    public function testReplace(): void
    {
        $productType = ProductType::factory()->createOne();
        $data = ProductType::factory()->makeFillableData();

        $response = $this->apiReplace('replaceProductType', $productType, $data);

        $response->assertStatus(200);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
        $response->assertJsonFragment($data);
    }

    public function testPatch(): void
    {
        $productType = ProductType::factory()->createOne();
        $data = ['name' => 'New patch name'];

        $response = $this->apiPatch('patchProductType', $productType, $data);

        $response->assertStatus(200);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
        $response->assertJsonFragment($data);
    }

    public function testDelete(): void
    {
        $productType = ProductType::factory()->createOne();

        $response = $this->apiDelete('deleteProductType', $productType);

        $response->assertStatus(200);
        $response->assertJsonFragment(['data' => null]);
        $this->assertNull(ProductType::find($productType->id));
    }

    public function testMassDelete(): void
    {
        $ids = ProductType::factory()
            ->count(3)
            ->create()
            ->pluck('id')
            ->toArray();

        $response = $this->apiMassDelete('deleteProductTypes', $ids);

        $response->assertStatus(200);
        $response->assertJsonFragment(['data' => null]);

        $this->assertCount(0, ProductType::find($ids));
    }

    public function testGet(): void
    {
        $productType = ProductType::factory()->createOne();

        $response = $this->apiGet('getProductType', $productType);

        $response->assertStatus(200);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
    }

    public function testSearch(): void
    {
        $productType = ProductType::factory()->count(3)->create()[1];

        $response = $this->apiSearch('searchProductTypes', ['code' => $productType->code]);

        $response->assertStatus(200);
        $response->assertJsonCount(1, 'data');
        $response->assertJsonFragment(['name' => $productType->name]);
    }
}
