<?php

namespace Tests\Feature;

use App\Domain\Classifiers\Models\Manufacturer;
use App\Http\ApiV1\Modules\Classifiers\Controllers\ManufacturersController;
use App\Http\Web\Controllers\HealthCheck;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Mockery;
use Mockery\LegacyMockInterface;
use Mockery\MockInterface;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Tests\TestCase;

class ErrorHandlerTest extends TestCase
{
    public function testModelNotFound(): void
    {
        $exception = (new ModelNotFoundException())->setModel(Manufacturer::class, [100, 120]);
        $this->mockController(ManufacturersController::class)
            ->shouldReceive('delete')->andThrow($exception);

        $response = $this->json(
            'DELETE',
            route('deleteManufacturer', ['entityId' => 100], false)
        );

        $response->assertStatus(404);
    }

    public function testHttpException(): void
    {
        $this->mockController(ManufacturersController::class)
            ->shouldReceive('delete')->andThrow(new AccessDeniedHttpException());

        $response = $this->json(
            'DELETE',
            route('deleteManufacturer', ['entityId' => 100], false)
        );

        $response->assertStatus(403);
    }

    public function testNotApi(): void
    {
        $this->mockController(HealthCheck::class)
            ->shouldReceive('__invoke')->andThrow(new AccessDeniedHttpException());

        $response = $this->get(route('healthCheck', [], false));

        $response->assertStatus(403);
        $response->assertHeader('content-type', 'text/html; charset=UTF-8');
    }

    /**
     * @param string $class
     * @return LegacyMockInterface|MockInterface
     */
    private function mockController(string $class)
    {
        $controller = Mockery::mock($class);
        $this->app->instance($class, $controller);

        return $controller;
    }
}
