<?php

namespace Tests\Feature\Products;

use App\Domain\Products\Models\Product;
use App\Domain\Products\Models\ProductImage;
use App\Domain\Products\Models\ProductImageType;
use Tests\APITestCase;

class ProductImagesTest extends APITestCase
{
    private static $jsonStructure = ['id', 'type', 'product_id', 'file'];

    public function testCreate(): void
    {
        $product = Product::factory()->createOne();
        $fields = [
            'product_id' => $product->id,
            'type' => ProductImageType::MAIN,
        ];

        $response = $this->apiCreate('createProductImage', $fields);

        $response->assertStatus(201);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
        $response->assertJsonFragment(['file' => null]);
    }

    public function testPatch(): void
    {
        $product = Product::factory()->createOne();
        $image = ProductImage::factory()->owned($product)->createOne([
            'type' => ProductImageType::MAIN(),
        ]);

        $data = ['type' => ProductImageType::CATALOG];

        $response = $this->json(
            'PATCH',
            $this->makeResourceApiUrl('patchProductImage', $image),
            $data
        );

        $response->assertStatus(200);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
        $response->assertJsonFragment($data);
    }

    public function testDelete(): void
    {
        $product = Product::factory()->createOne();
        $image = ProductImage::factory()->owned($product)->createOne();

        $response = $this->json(
            'DELETE',
            $this->makeResourceApiUrl('deleteProductImage', $image),
            []
        );

        $response->assertStatus(200);
        $this->assertDatabaseMissing($image->getTable(), ['id' => $image->id]);
    }

    public function testGet(): void
    {
        $product = Product::factory()->createOne();
        $image = ProductImage::factory()->owned($product)->createOne();

        $response = $this->apiGet('getProductImage', $image);

        $response->assertStatus(200);
        $response->assertJsonStructure(['data' => self::$jsonStructure]);
    }

    public function testSearch(): void
    {
        $product = Product::factory()->createOne();
        $image = ProductImage::factory()
            ->owned($product)
            ->count(2)
            ->create()[0];

        $response = $this->apiSearch('searchProductImages', ['product_id' => $product->id]);

        $response->assertStatus(200);
        $response->assertJsonCount(2, 'data');
        $response->assertJsonFragment(['id' => $image->id]);
    }
}
