<?php

namespace Tests\Integration\Api\Products\Requests;

use App\Domain\Categories\Models\Property;
use App\Domain\Categories\Models\PropertyType;
use App\Http\ApiV1\Modules\Products\Requests\ReplaceProductAttributesRequest;
use Illuminate\Foundation\Testing\DatabaseTransactions;
use Illuminate\Validation\ValidationException;
use Tests\TestCase;

class ReplaceProductAttributesRequestTest extends TestCase
{
    use DatabaseTransactions;

    /** @var Property */
    private $property;

    protected function setUp(): void
    {
        parent::setUp();

        $this->property = Property::factory()->createOne(['type' => PropertyType::INTEGER]);
    }

    /**
     * @dataProvider provideValues
     */
    public function testValues(array $source, array $expected): void
    {
        $data = [];
        foreach ($source as $item) {
            $data[] = ['property_id' => $this->property->id, 'value' => $item];
        }
        $request = $this->createRequest($data);

        $this->assertEquals([$this->property->id => $expected], $request->values());
    }

    public function provideValues(): array
    {
        return [
            'array with single value' => [[[11]], [11]],
            'array with multiple values' => [[[5, 10]], [5, 10]],
            'array with null value' => [[[null]], []],
            'empty array' => [[[]], []],
            'single value' => [[5], [5]],
            'null value' => [[null], []],
        ];
    }

    public function testValuesMultipleProperties(): void
    {
        $property2 = Property::factory()->createOne(['type' => PropertyType::STRING]);
        $data = [
            ['property_id' => $this->property->id, 'value' => 100],
            ['property_id' => $property2->id, 'value' => ['str']],
        ];
        $expected = [
            $this->property->id => [100],
            $property2->id => ['str'],
        ];

        $this->assertEquals($expected, $this->createRequest($data)->values());
    }

    private function createRequest(array $values): ReplaceProductAttributesRequest
    {
        try {
            return ReplaceProductAttributesRequest::createFromArray(['values' => $values], true, $this->app);
        } catch (ValidationException) {
            $this->fail('Validation failed');
        }
    }
}
