<?php

namespace Tests\Integration\Domain\Categories\Models;

use App\Domain\Categories\Models\Category;
use App\Domain\Categories\Models\CategoryPropertyLink;
use App\Domain\Categories\Models\Property;
use Illuminate\Foundation\Testing\DatabaseTransactions;
use Tests\TestCase;

class CategoryPropertyViewTest extends TestCase
{
    use DatabaseTransactions;

    public function testSelectEffectiveIds(): void
    {
        $parent = Category::factory()->createOne();
        $child = Category::factory()->withParent($parent)->createOne();
        $grandChild = Category::factory()->withParent($child)->createOne();

        $common = Property::factory()->createOne();
        $overridden = Property::factory()->createOne();
        $required = Property::factory()->createOne();

        $this->createLink($overridden, $parent);
        $this->createLink($required, $grandChild);

        $expected = [
            $this->createLink($common, $parent)->id,
            $this->createLink($overridden, $grandChild)->id,
            $this->createLink($required, $child, true)->id,
        ];

        $results = CategoryPropertyLink::selectEffectiveIds($grandChild->id);

        $this->assertEqualsCanonicalizing($expected, $results);
    }

    private function createLink(Property $property, Category $linkTo, bool $isRequired = false): CategoryPropertyLink
    {
        return CategoryPropertyLink::create([
            'category_id' => $linkTo->getKey(),
            'property_id' => $property->getKey(),
            'is_required' => $isRequired,
        ]);
    }
}
