<?php

namespace Tests\Integration\Domain\Products\Actions;

use App\Core\Elastic\IndexFactory;
use App\Core\Elastic\ScheduleRunner;
use App\Domain\Products\Actions\Products\MarkProductsForIndexAction;
use App\Domain\Products\Models\Product;
use Illuminate\Foundation\Testing\DatabaseTransactions;
use Illuminate\Support\Facades\Event;
use Laravie\SerializesQuery\Eloquent;
use PHPUnit\Framework\MockObject\MockObject;
use Tests\TestCase;

class MarkProductsForIndexActionTest extends TestCase
{
    use DatabaseTransactions;

    /** @var ScheduleRunner|MockObject */
    private $mockScheduler;

    /** @var MarkProductsForIndexAction */
    private $testing;

    protected function setUp(): void
    {
        parent::setUp();

        Event::fake();
        $this->mockScheduler = $this->createMock(ScheduleRunner::class);
        $this->testing = new MarkProductsForIndexAction($this->mockScheduler);
    }

    public function testExecute(): void
    {
        $products = Product::factory()->count(3)->create();
        $query = Product::query()->where('id', $products->first()->id);

        $this->mockScheduler->expects($this->once())->method('markForIndex');

        $this->testing->execute(Eloquent::serialize($query));
    }
}
