<?php

namespace Tests\Integration\Elastic\Products\Actions;

use App\Core\Elastic\Product\Indexing\Actions\FillPricesAction;
use App\Core\Elastic\Product\Indexing\OfferTicket;
use Ensi\MarketingClient\Api\DiscountsApi;
use Ensi\MarketingClient\Dto\CalculateForCatalogResponse;
use Ensi\MarketingClient\Dto\OffersDiscountCatalogResponse;
use Mockery;
use Mockery\LegacyMockInterface;
use Mockery\MockInterface;

class FillPricesActionTest extends OffersActionTestCase
{
    private DiscountsApi|LegacyMockInterface|MockInterface $mockApi;

    private FillPricesAction $testing;

    protected function setUp(): void
    {
        parent::setUp();

        $this->mockApi = Mockery::mock(DiscountsApi::class);
        $this->testing = new FillPricesAction($this->mockApi);
    }

    public function testExecute(): void
    {
        $ticket = $this->createOfferTicket(2, 160);

        $this->mockSearchResult([
            new OffersDiscountCatalogResponse(['offer_id' => 2, 'price' => 150, 'cost' => 160]),
        ]);

        $result = $this->executeOne($ticket);

        $result->assertActive();
        $result->assertPriceEquals(150);
    }

    public function testMissingBasePrice(): void
    {
        $ticket = $this->createOfferTicket(2);

        $this->mockApi->shouldNotReceive('calculateForCatalog');

        $result = $this->executeOne($ticket);

        $result->assertNotActive();
        $result->assertStrike('Нет цены');
    }

    public function testMissingResultPrice(): void
    {
        $ticket = $this->createOfferTicket(2, 160);
        $this->mockSearchResult([]);

        $result = $this->executeOne($ticket);

        $result->assertNotActive();
        $result->assertStrike('Нет цены');
    }

    private function mockSearchResult(array $prices): void
    {
        $this->mockApi->allows('calculateForCatalog')
            ->andReturn(new CalculateForCatalogResponse(['data' => $prices]));
    }

    private function executeOne(OfferTicket $ticket): TestOfferTicket
    {
        $this->testing->execute(collect([$ticket->id => $ticket]));

        return new TestOfferTicket($ticket);
    }
}
