<?php

namespace Tests\Integration\Elastic\Products\Actions;

use App\Core\Elastic\Product\Indexing\Actions\FillStocksAction;
use App\Core\Elastic\Product\Indexing\OfferTicket;
use Ensi\OffersClient\Api\StocksApi;
use Ensi\OffersClient\Dto\SearchStocksResponse;
use Ensi\OffersClient\Dto\Stock;
use Mockery;
use Mockery\LegacyMockInterface;
use Mockery\MockInterface;

class FillStocksActionTest extends OffersActionTestCase
{
    private StocksApi|LegacyMockInterface|MockInterface $mockApi;

    private FillStocksAction $testing;

    protected function setUp(): void
    {
        parent::setUp();

        $this->mockApi = Mockery::mock(StocksApi::class);
        $this->testing = new FillStocksAction($this->mockApi);
    }

    public function testExecute(): void
    {
        $ticket = $this->createOfferTicket(1);
        $this->mockSearchResult([
            new Stock(['offer_id' => 1, 'store_id' => 15, 'qty' => 8]),
        ]);

        $result = $this->executeOne($ticket);

        $result->assertActive();
        $result->assertSaleActive();
        $this->assertEquals(15, $result->storeId);
        $this->assertEquals(8, $result->qty);
    }

    public function testNoStocks(): void
    {
        $ticket = $this->createOfferTicket(1);
        $this->mockSearchResult([]);

        $result = $this->executeOne($ticket);

        $result->assertStrike('Нет остатков');
    }

    public function testNoStore(): void
    {
        $ticket = $this->createOfferTicket(1);
        $this->mockSearchResult([
            new Stock(['offer_id' => 1, 'qty' => 8]),
        ]);

        $result = $this->executeOne($ticket);

        $result->assertStrike('Нет склада');
    }

    public function testMinStock(): void
    {
        $ticket = $this->createOfferTicket(1);
        $this->mockSearchResult([
            new Stock(['offer_id' => 1, 'qty' => FillStocksAction::MIN_STOCK - 1]),
        ]);

        $result = $this->executeOne($ticket);

        $result->assertNotActive();
    }

    private function mockSearchResult(array $stocks): void
    {
        $this->mockApi->allows('searchStocks')
            ->andReturn(new SearchStocksResponse(['data' => $stocks]));
    }

    private function executeOne(OfferTicket $ticket): TestOfferTicket
    {
        $this->testing->execute(collect([$ticket->id => $ticket]));

        return new TestOfferTicket($ticket);
    }
}
