<?php

namespace Tests\Integration\Elastic\Products\Actions;

use App\Core\Elastic\Product\Indexing\OfferTicket;
use App\Core\Elastic\Product\OfferIndexProperty;
use Illuminate\Support\Traits\ForwardsCalls;
use PHPUnit\Framework\Assert;

/**
 * @mixin OfferIndexProperty
 */
class TestOfferTicket
{
    use ForwardsCalls;

    private ?array $result;

    public function __construct(private OfferTicket $ticket)
    {
    }

    public function __call(string $name, array $arguments)
    {
        return $this->forwardCallTo($this->ticket->target, $name, $arguments);
    }

    public function __get(string $name)
    {
        return $this->ticket->target->{$name};
    }

    public function __set(string $name, $value): void
    {
        $this->ticket->target->{$name} = $value;
    }

    public function toArray(): array
    {
        $this->result ??= $this->ticket->target->toArray();

        return $this->result;
    }

    public function assertStrike(string $expected): void
    {
        Assert::assertContains($expected, $this->toArray()['strikes'] ?? []);
    }

    public function assertActive(): void
    {
        Assert::assertTrue($this->toArray()['active'] ?? false);
    }

    public function assertNotActive(): void
    {
        Assert::assertFalse($this->toArray()['active'] ?? false);
    }

    public function assertSaleActive(): void
    {
        Assert::assertTrue($this->toArray()['saleActive'] ?? false);
    }

    public function assertNotSaleActive(): void
    {
        Assert::assertFalse($this->toArray()['saleActive'] ?? false);
    }

    public function assertPriceEquals(int $expected): void
    {
        Assert::assertEquals($expected, $this->ticket->target->price['price'] ?? null);
    }
}
