<?php

namespace Tests\Integration\Elastic\Products\Stages;

use App\Core\Elastic\Product\Indexing\Stages\AttributesStage;
use App\Domain\Categories\Models\Property;
use App\Domain\Categories\Models\PropertyType;
use App\Domain\Products\Models\Product;
use App\Domain\Products\Models\ProductPropertyValue;

class AttributesStageTest extends StageTestCase
{
    public function testProperty(): void
    {
        $product = Product::factory()->createOne();
        $property = Property::factory()->createOne(['type' => PropertyType::STRING]);
        $this->addPropertyValue($product, $property, 'Foo');

        $result = $this->executeOne(new AttributesStage(), $product);

        $result->assertNotFailed();
        $result->assertAttributeEquals('Foo', $property->name);
    }

    public function testMultiValueProperty(): void
    {
        $product = Product::factory()->createOne();
        $property = Property::factory()->createOne(['type' => PropertyType::STRING]);
        $this->addPropertyValue($product, $property, 'Foo');
        $this->addPropertyValue($product, $property, 'Bar');

        $result = $this->executeOne(new AttributesStage(), $product);

        $result->assertAttributeEquals(['Foo', 'Bar'], $property->name);
    }

    public function testDirectoryProperty(): void
    {
        $product = Product::factory()->createOne();
        $property = Property::factory()->directory()->createOne();
        $this->addDirectoryValue($product, $property, 'Foo');

        $result = $this->executeOne(new AttributesStage(), $product);

        $result->assertNotFailed();
        $result->assertAttributeEquals('Foo', $property->name);
    }

    public function testFilterableProperty(): void
    {
        $product = Product::factory()->createOne();
        $property = Property::factory()->createOne(['type' => PropertyType::STRING, 'is_filterable' => true]);
        $this->addPropertyValue($product, $property, 'Foo');

        $result = $this->executeOne(new AttributesStage(), $product);

        $this->assertContains("$property->code#Foo", $result->props);
    }

    public function testField(): void
    {
        $product = Product::factory()->createOne(['weight' => 1168.0]);

        $result = $this->executeOne(new AttributesStage(), $product);

        $result->assertAttributeEquals(1168.0, 'Масса нетто, кг');
    }

    private function addPropertyValue(Product $product, Property $property, mixed $value): void
    {
        $propertyValue = ProductPropertyValue::make($product, $property, $value);
        $propertyValue->save();
    }

    private function addDirectoryValue(Product $product, Property $property, string $name): void
    {
        $directory = $property->directory()
            ->create(['property_id' => $property->id, 'name' => $name]);

        $this->addPropertyValue($product, $property, $directory->getKey());
    }
}
