<?php

namespace Tests\Integration\Elastic\Products\Stages;

use App\Core\Elastic\Product\Indexing\ProductTicket;
use App\Core\Elastic\Product\ProductIndexRecord;
use Illuminate\Contracts\Support\Arrayable;
use Illuminate\Support\Arr;
use Illuminate\Support\Traits\ForwardsCalls;
use PHPUnit\Framework\Assert;

/**
 * @mixin ProductIndexRecord
 */
class TestProductTicket implements Arrayable
{
    use ForwardsCalls;

    private ?array $result;

    public function __construct(private ProductTicket $ticket)
    {
    }

    public function __call(string $name, array $arguments)
    {
        return $this->forwardCallTo($this->ticket->target, $name, $arguments);
    }

    public function __get(string $name)
    {
        return $this->ticket->target->{$name};
    }

    public function __set(string $name, $value): void
    {
        $this->ticket->target->{$name} = $value;
    }

    public function toArray(): array
    {
        $this->result ??= $this->ticket->target->toArray();

        return $this->result;
    }

    public function assertActive(): void
    {
        Assert::assertTrue($this->toArray()['active'] ?? false);
    }

    public function assertNotActive(): void
    {
        Assert::assertFalse($this->toArray()['active'] ?? false);
    }

    public function assertFailed(): void
    {
        Assert::assertTrue($this->ticket->isFailed);
    }

    public function assertNotFailed(): void
    {
        Assert::assertFalse($this->ticket->isFailed);
    }

    public function assertAttributeEquals(mixed $expected, string $name): void
    {
        $attribute = Arr::first(
            $this->toArray()['characteristics'] ?? [],
            fn(array $item) => $item['name'] === $name
        );

        Assert::assertNotNull($attribute);
        Assert::assertEqualsCanonicalizing((array)$expected, $attribute['value']);
    }

    public function assertHasStrike(string $expected): void
    {
        Assert::assertContains($expected, $this->toArray()['strikes'] ?? []);
    }
}
