<?php

namespace Tests\Unit;

use Illuminate\Support\Facades\Date;
use PHPUnit\Framework\TestCase;

class HelpersTest extends TestCase
{
    /**
     * @param array $values
     * @param mixed $expected
     * @param string $message
     * @dataProvider provideCoalesce
     */
    public function testCoalesce(array $values, $expected, string $message): void
    {
        $this->assertEquals($expected, coalesce(...$values), $message);
    }

    public function provideCoalesce(): array
    {
        return [
            [[1, 2], 1, 'Выбран не первый непустой элемент'],
            [[null, '', ' ', 'val'], 'val', 'Выбран пустой элемент'],
            [[Date::make('2021-02-20'), 1], Date::make('2021-02-20'), 'Пропущен объект'],
        ];
    }

    public function testDataCombineAssoc(): void
    {
        $values = ['one' => 1, 'two' => 2];
        $keys = ['one', 'three'];

        $this->assertEquals(
            ['one' => 1, 'three' => null],
            data_combine_assoc($keys, $values)
        );
    }

    public function testDataCombineAssocEmptyKeys(): void
    {
        $this->assertEquals([], data_combine_assoc([], ['one' => 1]));
    }

    public function testDataCombineAssocDefaultValue(): void
    {
        $this->assertEquals(
            ['one' => 'value', 'two' => 'value'],
            data_combine_assoc(['one', 'two'], [], 'value')
        );
    }
}
