<?php

namespace Tests\Unit\Support\Files;

use App\Support\Files\FileIntent;
use Illuminate\Support\Str;
use Tests\UnitTestCase;

class FileIntentTest extends UnitTestCase
{
    public function testConstructGenerateUid(): void
    {
        $this->assertNotEquals(FileIntent::fake()->getUid(), FileIntent::fake()->getUid());
    }

    public function testGetName(): void
    {
        $intent = FileIntent::fake();

        $this->assertEquals($intent->getUid().'.png', $intent->getName());
    }

    public function testGetNameWithPrefix(): void
    {
        $intent = FileIntent::fake()->withNamePrefix('brand_11');

        $this->assertEquals('brand_11_'.$intent->getUid().'.png', $intent->getName());
    }

    public function testGetDirectory(): void
    {
        $intent = FileIntent::fake();

        $this->assertCount(2, explode('/', $intent->getDirectory()));
    }

    public function testGetDirectoryWithParent(): void
    {
        $intent = FileIntent::fake()->inDirectory('evolution');

        $this->assertCount(3, explode('/', $intent->getDirectory()));
        $this->assertTrue(Str::startsWith($intent->getDirectory(), 'evolution/'));
    }
}
