<?php

namespace Tests\Unit\Support\Files;

use App\Exceptions\FileStorageException;
use App\Support\Files\FileIntent;
use App\Support\Files\FileStorage;
use Illuminate\Contracts\Filesystem\Cloud;
use Mockery;
use Tests\UnitTestCase;

class FileStorageTest extends UnitTestCase
{
    /** @var FileStorage */
    private $testing;

    /** @var Cloud|Mockery\LegacyMockInterface|Mockery\MockInterface */
    private $mockFileSystem;
    private string $fakePath;

    protected function setUp(): void
    {
        parent::setUp();

        $this->fakePath = "path/to/test1.png";
        $this->mockFileSystem = Mockery::mock(Cloud::class);
        $this->testing = new FileStorage($this->mockFileSystem);
    }

    protected function tearDown(): void
    {
        Mockery::close();

        parent::tearDown();
    }

    public function testAdd(): void
    {
        $this->mockFileSystem->shouldReceive('putFileAs')->andReturn('path/to/file/logo1.png');

        $intent = FileIntent::fake('logo1.png');

        $filePath = $this->testing->add($intent);

        $this->assertEquals('path/to/file/logo1.png', $filePath);
    }

    public function testAddError(): void
    {
        $this->expectException(FileStorageException::class);

        $this->mockFileSystem->shouldReceive('putFileAs')->andReturn(false);
        $intent = FileIntent::fake('logo1.png');

        $this->testing->add($intent);
    }

    public function testRemove(): void
    {
        $this->mockFileSystem->shouldReceive('exists')->andReturn(true);
        $this->mockFileSystem->shouldReceive('delete')
            ->with($this->fakePath)
            ->andReturn(true)
            ->once();

        $this->testing->remove($this->fakePath);

        $this->assertTrue(true);
    }

    public function testRemoveNotExisting(): void
    {
        $this->mockFileSystem->shouldReceive('exists')->andReturn(false);

        $this->testing->remove($this->fakePath);

        $this->assertTrue(true);    // Выполнено без ошибок
    }

    public function testRemoveError(): void
    {
        $this->expectException(FileStorageException::class);

        $this->mockFileSystem->shouldReceive('exists')->andReturn(true);
        $this->mockFileSystem->shouldReceive('delete')->andReturn(false);

        $this->testing->remove($this->fakePath);
    }
}
