<?php

namespace Tests\Unit\Support\Files;

use App\Support\Files\FileIntent;
use App\Support\Files\FileStorage;
use App\Support\Files\LinksFile;
use Mockery;
use RuntimeException;
use Tests\UnitTestCase;

class LinksFileTest extends UnitTestCase
{
    /** @var LinkFileTestStub */
    private $testing;

    /** @var FileStorage|Mockery\LegacyMockInterface|Mockery\MockInterface */
    private $mockStorage;
    private string $fakePath;

    protected function setUp(): void
    {
        parent::setUp();

        $this->fakePath = "path/to/test1.png";
        $this->mockStorage = Mockery::mock(FileStorage::class);
        $this->testing = new LinkFileTestStub($this->mockStorage);
    }

    protected function tearDown(): void
    {
        Mockery::close();

        parent::tearDown();
    }

    public function testLink(): void
    {
        $intent = FileIntent::fake();

        $callback = function (string $filePath) {
            $this->assertSame($this->fakePath, $filePath);
        };

        $this->mockStorage->shouldReceive('add')
            ->with($intent)
            ->andReturn($this->fakePath)
            ->once();

        $this->testing->link($intent, $callback);
    }

    public function testLinkRemovesOld(): void
    {
        $intent = FileIntent::fake();
        $filePathOld = 'path/to/file/test_old.jpg';

        $callback = function () use ($filePathOld) {
            return $filePathOld;
        };

        $this->mockStorage->shouldReceive('add')->andReturn($this->fakePath);
        $this->mockStorage->shouldReceive('remove')
            ->with($filePathOld)
            ->once();

        $this->testing->link($intent, $callback);

        $this->assertTrue(true);
    }

    public function testLinkFail(): void
    {
        $intent = FileIntent::fake();

        $callback = function () {
            throw new RuntimeException();
        };

        $this->mockStorage->shouldReceive('add')->andReturn($this->fakePath);
        $this->mockStorage->shouldReceive('remove')
            ->with($this->fakePath)
            ->once();

        $this->expectException(RuntimeException::class);

        $this->testing->link($intent, $callback);
    }

    public function testUnlink(): void
    {
        $callback = function () {
            return $this->fakePath;
        };

        $this->mockStorage->shouldReceive('remove')
            ->with($this->fakePath)
            ->once();

        $this->testing->unlink($callback);

        $this->assertTrue(true);
    }
}

class LinkFileTestStub
{
    use LinksFile {
        link as public;
        unlink as public;
    }

    public function __construct(FileStorage $storage)
    {
        $this->storage = $storage;
    }
}
